dnl dancer-c.m4
dnl This file is copyright (C) 2001 Andrew Suffield
dnl                                  <asuffield@users.sourceforge.net>
dnl
dnl This program is free software; you can redistribute it and/or modify
dnl it under the terms of the GNU General Public License as published by
dnl the Free Software Foundation; either version 2 of the License, or
dnl (at your option) any later version.
dnl
dnl This program is distributed in the hope that it will be useful,
dnl but WITHOUT ANY WARRANTY; without even the implied warranty of
dnl MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
dnl GNU General Public License for more details.
dnl
dnl You should have received a copy of the GNU General Public License
dnl along with this program; if not, write to the Free Software
dnl Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA

dnl Stolen from dpkg

dnl DANCER_C_GCC_TRY_FLAGS(<warnings>)
AC_DEFUN(DANCER_C_GCC_TRY_FLAGS,[
 AC_MSG_CHECKING([GCC warning flag(s) $1])
 if test "${GCC-no}" = yes
 then
   oldcflags="${CFLAGS-}"
   CFLAGS="${CFLAGS-} ${CWARNS} $1 -Werror"
   AC_TRY_COMPILE([
#include <string.h>
#include <stdio.h>

const char *a = "a";

const char *foo(void);
inline const char *foo(void) {return a;}

int main(void);
],[
    strcmp(foo(),"b"); fprintf(stdout,"test ok\n");
],[
   CWARNS="${CWARNS}$1 "
   AC_MSG_RESULT(ok)
],[
   AC_MSG_RESULT(no)
])
   CFLAGS="${oldcflags}"
 else
  AC_MSG_RESULT(not using GCC, skipping)
 fi
])

AC_DEFUN(DANCER_C_GCC_LD_TRY_FLAGS,[
 AC_MSG_CHECKING([GCC warning flag(s) $1])
 if test "${GCC-no}" = yes
 then
   oldldflags="${LDFLAGS-}"
   LDFLAGS="${LDFLAGS-} ${LDWARNS} -Wl,$1 -Wl,--fatal-warnings"
   AC_TRY_LINK([
#include <string.h>
#include <stdio.h>

const char *a = "a";

const char *foo(void);
inline const char *foo(void) {return a;}

int main(void);
],[
    strcmp(foo(),"b"); fprintf(stdout,"test ok\n");
],[
   LDWARNS="${LDWARNS}-Wl,$1 "
   AC_MSG_RESULT(ok)
],[
   AC_MSG_RESULT(no)
])
   LDFLAGS="${oldldflags}"
 else
  AC_MSG_RESULT(not using GCC, skipping)
 fi
])

AC_DEFUN(DANCER_CHECK_LONG_LONG,
[AC_MSG_CHECKING(for long long type and printf support)
oldcflags="${CFLAGS-}"
CFLAGS="${CFLAGS-} -Werror"
AC_TRY_COMPILE([
#include <stdio.h>
int main(void);
],[
long long int x;
x = 1;
x += 1;
printf("%lld",x);
],AC_DEFINE(HAVE_LONG_LONG,,[gcc extension type long long is available]) AC_MSG_RESULT(yes), AC_MSG_RESULT(no))
CFLAGS="${oldcflags}"
])

dnl We are only interested in __attribute__((format(printf,,))) if it can handle %zd
AC_DEFUN(DANCER_CHECK_C_ATTRIBUTE_PRINTF_ZD,
[AC_MSG_CHECKING([for gcc __attribute__(()) format printf %zd support])
oldcflags="${CFLAGS-}"
CFLAGS="${CFLAGS-} -Werror"
AC_TRY_COMPILE([
extern  void foo(const char *, ...)
     __attribute__((format(printf,1,2)));
int main(void);
],[
foo("%zi",sizeof(int));
],AC_DEFINE(HAVE_ATTRIBUTE,,[gcc-style attribute declarations available]) AC_MSG_RESULT(yes), AC_MSG_RESULT(no))
CFLAGS="${oldcflags}"
])

dnl See if the named attribute flag is available and create a suitable #define if necessary
AC_DEFUN(DANCER_CHECK_C_ATTRIBUTE,
[AC_MSG_CHECKING([for __attribute__(($1))])
oldcflags="${CFLAGS-}"
CFLAGS="${CFLAGS-} -Werror"
AC_TRY_COMPILE([
extern void *foo(void) __attribute__(($1));
int main(void);
],[
foo();
],
attribute_value="__attribute__(($1))"
AC_MSG_RESULT(yes),
attribute_value=""
AC_MSG_RESULT(no))
CFLAGS="${oldcflags}"
AC_DEFINE_UNQUOTED([$1_attribute], $attribute_value, [Support for __attribute__(($1))])
])

AC_DEFUN([DANCER_C_INLINE],
[AC_REQUIRE([AC_PROG_CC_STDC])dnl
oldcflags="${CFLAGS-}"
CFLAGS="${CFLAGS-} -Werror"
AC_CACHE_CHECK([for inline], ac_cv_c_inline,
[ac_cv_c_inline=no
for ac_kw in inline __inline__ __inline; do
  AC_COMPILE_IFELSE([AC_LANG_SOURCE(
[#ifndef __cplusplus
static $ac_kw int static_foo (void) {return 0; }
int foo(void);
$ac_kw int foo (void) {return 0; }
#endif
])],
                    [ac_cv_c_inline=$ac_kw; break])
done
])
case $ac_cv_c_inline in
  inline | yes) ;;
  no) AC_DEFINE(inline,,
                [Define as `__inline' if that's what the C compiler calls it,
                 or to nothing if it is not supported.]) ;;
  *)  AC_DEFINE_UNQUOTED(inline, $ac_cv_c_inline) ;;
esac
CFLAGS="${oldcflags}"
])

AC_DEFUN(DANCER_C_CONST,
[oldcflags="${CFLAGS-}"
CFLAGS="${CFLAGS-} -Werror"
AC_C_CONST
CFLAGS="${oldcflags}"
])

AC_DEFUN(DANCER_C_MACRO_PRAGMA,
[AC_MSG_CHECKING([for working _Pragma])
oldcflags="${CFLAGS-}"
CFLAGS="${CFLAGS-} -Werror"
AC_TRY_COMPILE([
#define ALPHA
#define BETA(X) int z ## X
#define P(X) _Pragma("foo"); ALPHA BETA(X); int x
],[
P(y);
x = 1;
],
AC_DEFINE(HAVE_MACRO_PRAGMA, 1, [_Pragma works when used in a macro])
AC_MSG_RESULT(yes),
AC_MSG_RESULT(no))
])
