dnl dancer-headers.m4
dnl This file is copyright (C) 2001 Andrew Suffield
dnl                                  <asuffield@users.sourceforge.net>
dnl
dnl This program is free software; you can redistribute it and/or modify
dnl it under the terms of the GNU General Public License as published by
dnl the Free Software Foundation; either version 2 of the License, or
dnl (at your option) any later version.
dnl
dnl This program is distributed in the hope that it will be useful,
dnl but WITHOUT ANY WARRANTY; without even the implied warranty of
dnl MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
dnl GNU General Public License for more details.
dnl
dnl You should have received a copy of the GNU General Public License
dnl along with this program; if not, write to the Free Software
dnl Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA

dnl This evil magic tests for broken libc headers that cause compiler warnings

AC_DEFUN(DANCER_HEADERS,[

dnl This is the list of all system headers that are included in dancer. If libc can
dnl compile with all these included and compiler errors on, then it is considered to
dnl be safe to use those errors.
dnl
dnl System headers are any includes using <foo> rather than "foo", and which do not
dnl have a <dancerd/foo> prefix.

AC_MSG_CHECKING([for system headers used])
dancer_headers_used=`find ${srcdir} -path ./\{arch\} -prune -or \( -name '*.c' -or -name '*.h' \) -print | xargs egrep -r -h '^\# *include +<' | egrep -v '^\# *include +<dancerd/' | sed 's/>.*/>/' | sed 's/\# */\#/' | sed 's/include *</include </' | sort | uniq`
dancer_headers_count=`printf "$dancer_headers_used" | wc -l | sed 's/^ *//'`
AC_MSG_RESULT([found $dancer_headers_count])

dnl OK, get the simple list to check for existance
dancer_headers_list=`printf "$dancer_headers_used" | sed 's/\#include <\(.*\)>/\1/'`
AC_CHECK_HEADERS($dancer_headers_list)

dnl Now compose the working list of guarded headers...
dancer_headers_guarded=`printf "$dancer_headers_list\n" | sed "h;y%abcdefghijklmnopqrstuvwxyz./-%ABCDEFGHIJKLMNOPQRSTUVWXYZ___%;s/\(.*\)/\#ifdef HAVE_\1/;x;s/\(.*\)/\#include <\1>/;x;G;a\\\\
\#endif
"`
])

AC_DEFUN(DANCER_CHECK_LIBC_PEDANTIC,[
dnl OK, can we compile with -pedantic-errors, or is libc broken?
dancer_check_libc_pedantic_save_CFLAGS="$CFLAGS"
CFLAGS="$CFLAGS -pedantic-errors"
AC_TRY_COMPILE([
$dancer_headers_guarded

void foo(void);
inline void foo(void) {}

int main(void);
],[
  foo();
  return 0;
], [$1], [$2])
CFLAGS="$dancers_check_libc_pedantic_save_CFLAGS"])

AC_DEFUN(DANCER_CHECK_LIBC_WERROR,[
dnl <sigh> Can we compile with -Werror, or is libc broken?
dancer_save_CFLAGS="$CFLAGS"
CFLAGS="$CFLAGS -Werror"
AC_TRY_COMPILE([
$dancer_headers_guarded

void foo(void);
inline void foo(void) {}

int main(void);
],[
  foo();
  return 0;
], [$1], [$2])
CFLAGS="$dancer_save_CFLAGS"
])
