/************************************************************************
 *   This is part of Dancer, a text relay daemon
 *   Copyright (C) 2002 Andrew Suffield <asuffield@users.sourceforge.net>
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 */

#ifdef CLOCK_H
#else
#define CLOCK_H
#include <dancerd/config.h>

#if TIME_WITH_SYS_TIME
# include <sys/time.h>
# include <time.h>
#else
# if HAVE_SYS_TIME_H
#  include <sys/time.h>
# else
#  include <time.h>
# endif
#endif

/** \file clock.h 
 * \brief Real-time clock
 */

/** \defgroup clock Real-time clock
 * \ingroup main
 * Caches current time to avoid expensive time() syscalls, and
 * provides a single interface to get the current time
 *
 * @{
 */

/** \brief Contains time since the epoch
 */
struct time_hires
{
  /** Time since the epoch in seconds */
  long int sec;
  /** Time since the start of the last second, in microseconds */
  long int usec;
};

extern time_t current_time;
extern struct time_hires current_time_hires;

extern time_t get_real_time(void);
extern void get_real_time_hires(struct time_hires *);
extern void update_clock(void);
extern const char *get_real_time_str(void);

/** @}
 */

#endif
