/************************************************************************
 *   This is part of Dancer, a text relay daemon
 *   Copyright (C) 2002 Andrew Suffield <asuffield@users.sourceforge.net>
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 */

#ifdef CTOR_H
typedef void constructor_func(void);
typedef void destructor_func(void);
#else
#define CTOR_H
#include <dancerd/config.h>

/** \file ctor.h
 * \brief Global constructors and destructors in the main binary
 */

/** \defgroup ctor Global constructors and destructors in the main binary
 * \ingroup mainguts
 * ctor provides a global construction/destruction system with
 * dependencies, so that ctors/dtors are called in the correct order.
 *
 * This should not be used in modules; instead, let the module
 * dependencies themselves sort it out.
 *
 * @{
 */

typedef void constructor_func(void);
typedef void destructor_func(void);
extern void register_constructor(constructor_func *, const char *, const char *[]);
extern void register_destructor(destructor_func *, const char *, const char *[]);

extern void do_global_construction(void);
extern void do_global_destruction(void);

#define GLOBAL_CONSTRUCTOR(NAME, ...) \
 static void _dancer__constructor__ ## NAME ## _(void); \
 static void _dancer__constructor__ ## NAME ## _register_(void) __attribute__((constructor)); \
 static void _dancer__constructor__ ## NAME ## _register_(void) \
 { \
   const char *deps[] = {__VA_ARGS__}; \
   register_constructor(&(_dancer__constructor__ ## NAME ## _), #NAME, deps); \
 } \
 static void _dancer__constructor__ ## NAME ## _(void)

/* Note that global destructors are registered when the daemon starts up */

#define GLOBAL_DESTRUCTOR(NAME, ...) \
 static void _dancer__destructor__ ## NAME ## _(void); \
 static void _dancer__destructor__ ## NAME ## _register_(void) __attribute__((constructor)); \
 static void _dancer__destructor__ ## NAME ## _register_(void) \
 { \
   const char *deps[] = {__VA_ARGS__}; \
   register_destructor(&(_dancer__destructor__ ## NAME ## _), #NAME, deps); \
 } \
 static void _dancer__destructor__ ## NAME ## _(void)

/** @} */

#endif
