/************************************************************************
 *   This is part of Dancer, a text relay daemon
 *   Copyright (C) 2002 Andrew Suffield <asuffield@users.sourceforge.net>
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 */

#ifdef FDBUFFER_H

struct fdbuffer;

#else
#define FDBUFFER_H
#include <dancerd/config.h>
#include <dancerd/dbuffer.h>
#include <dancerd/modules.h>

#include <sys/types.h>

/** \file fdbuffer.h
 * \brief Buffered fd access
 */

/** \defgroup fdbuffer Buffered fd access
 * \ingroup main
 * An fdbuffer manages an fd, registering it with the fd polling
 * system and reading to/writing from dbuffers in a FIFO manner.
 *
 * @{
 */ 

struct fdbuffer;

/* This gets (fdbuffer, dbuffer, data, len) where len bytes starting
 * at data have just been added to dbuffer
 *
 * When the socket is closed, it will be called with (data,len) = (NULL,0)
 */
typedef void fdbuffer_read_handler(struct fdbuffer *, struct dbuffer *, const char *, size_t);

extern struct fdbuffer *setup_fdbuffer(struct module *, int, int, fdbuffer_read_handler *);
extern void destroy_fdbuffer(struct fdbuffer *);
extern void write_fdbuffer(struct fdbuffer *, const char *, size_t);
extern size_t fdbuffer_write_pending(struct fdbuffer *);
/* This returns true if the buffer is closed */
extern int fdbuffer_closed(struct fdbuffer *);

extern void attach_fdbuffer_read_handler(struct module *, struct fdbuffer *, fdbuffer_read_handler *);

extern void printf_fdbuffer(struct fdbuffer *fdb, const char *format, ...) printf_format_attribute(2,3);

/** @}
 */

#endif
