/************************************************************************
 *   This is part of Dancer, a text relay daemon
 *   Copyright (C) 2002 Andrew Suffield <asuffield@users.sourceforge.net>
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 */

#ifdef HASHTABLE_H
struct hashtable;
#else
#define HASHTABLE_H
#include <dancerd/config.h>

#include <stdint.h>

/** \file hashtable.h
 * \brief Hash table, keyed by strings (struct hashtable)
 */

/** \defgroup hashtable Hash table, keyed by strings (struct hashtable)
 * \ingroup datastructure
 * This is a moderately efficient, generic hash table
 * implementation. Keys are null-terminated strings, values are void*.
 *
 * @{
 */ 

struct hashtable;

typedef uint32_t hash_func(const char *);

struct hashtable *new_hashtable(hash_func *, uint32_t);
extern void free_hashtable(struct hashtable *);

extern void hashtable_insert(struct hashtable *, const char *, void *);
extern void *hashtable_lookup(struct hashtable *, const char *);
extern void hashtable_delete(struct hashtable *, const char *);
extern int hashtable_check_sanity(struct hashtable *);

hash_func hash_ascii;

/** @}
 */

#endif
