/************************************************************************
 *   This is part of Dancer, a text relay daemon
 *   Copyright (C) 2002 Andrew Suffield <asuffield@users.sourceforge.net>
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 */

#ifdef SIGNAL_HANDLER_H

typedef void signal_handler_func(int, void *);

#else
#define SIGNAL_HANDLER_H
#include <dancerd/config.h>

/* Include this here to pick up the signal numbers */
#include <signal.h>

/** \file signal_handler.h
 * \brief Register signal handlers
 */

/** \defgroup signalhandler Register signal handlers
 * \ingroup main
 * Provides a mechanism for registering safe signal handlers. Signal
 * handlers registered here will be called from the top level of the
 * main polling loop, and not from anywhere else, so they are safe and
 * do not cause re-entrancy issues.
 *
 * @{
 */ 

typedef void signal_handler_func(int, void *);

extern void poll_signals(void);
extern void register_signal_handler(int, signal_handler_func *, void *);
extern void deregister_signal_handler(int, signal_handler_func *, void *);

/** @}
 */

#endif
