/************************************************************************
 *   This is part of Dancer, a text relay daemon
 *   Copyright (C) 2002 Andrew Suffield <asuffield@users.sourceforge.net>
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 */

#ifdef TIMER_H
#else
#define TIMER_H
#include <dancerd/config.h>
#include <dancerd/clock.h>

/** \file timer.h
 * \brief Register callbacks to be triggered at specific times
 */

/** \defgroup timer Register callbacks to be triggered at specific times
 * \ingroup main
 * The timer will call registered handlers as soon as possible after
 * the given time, from the top level of a the polling loop.
 *
 * This means that there are no re-entrancy issues, and timers will
 * not interrupt a function in a module.
 *
 * @{
 */ 

typedef void timer_handler_func(void *);

extern void poll_timer(void);
extern void register_timer(time_t, timer_handler_func *, void *);
extern void register_timer_hires(struct time_hires *, timer_handler_func *, void *);

/** @}
 */

#endif
