/************************************************************************
 *   This is part of Dancer, a text relay daemon
 *   Copyright (C) 2002 Andrew Suffield <asuffield@users.sourceforge.net>
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 */

#ifdef XMALLOC_H
#else
#define XMALLOC_H
#include <dancerd/config.h>

#include <sys/types.h>
#include <stdlib.h>
/* Include string.h so it isn't affected by the malloc poison */
#include <string.h>

/** \file xmalloc.h
 * \brief Reliable malloc wrapper
 */

/** \defgroup xmalloc Reliable malloc wrapper
 * \ingroup main
 * The old classic set of functions, these behave like their
 * counterparts except that they never return NULL; if not enough
 * memory is available, they do not return.
 *
 * @{
 */ 

extern void out_of_memory(void) noreturn_attribute;
/* xmalloc will allocate the given number of bytes of memory
 *         will not return NULL
 */
extern void *xmalloc(size_t) malloc_attribute;
extern void *xcalloc(size_t) malloc_attribute;
extern void *xrealloc(void *, size_t) malloc_attribute;
static inline void xfree(void *) unused_attribute;
static inline void xfree(void *x) {free(x);}

#ifndef DANCER_CORE__XMALLOC_SOURCE__
# ifdef malloc
#  undef malloc
# endif
# ifdef free
#  undef free
# endif
# ifdef realloc
#  undef realloc
# endif
# pragma GCC poison malloc free realloc
#endif

/** @}
 */

#endif
