/************************************************************************
 *   This is part of Dancer, a text relay daemon
 *   Copyright (C) 2002 Andrew Suffield <asuffield@users.sourceforge.net>
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 */

#ifdef XSTRING_H
#else
#define XSTRING_H
#include <dancerd/config.h>

#include <sys/types.h>
#include <string.h>
#include <stdio.h>

#include <dancerd/xmalloc.h>

/** \file xstring.h
 * \brief Safer replacement for some of the functions in string.h
 */

/** \defgroup xstring Safer replacement for some of the functions in string.h
 * \ingroup main
 * A set of replacements for some of the classic C string manipulations.
 *
 * @{
 */ 

/* xstrndup will copy up to the given number of bytes from the given string, plus a terminating null
 *          will allocate the given number of bytes plus one
 *          will stop at the first null in the original data
 *
 * xstrdup(NULL) == xstrndup(NULL, x) == NULL
 */
extern void *xstrndup(const char *, size_t);
static inline void *xstrdup(const char *) unused_attribute;
static inline void *xstrdup(const char *s) {return s ? xstrndup(s, strlen(s) + 1) : NULL;}
/* xmemdup will allocate a new region of size l, then copy l bytes from d into it */
static inline void *xmemdup(void *d, size_t l) {void *n = xmalloc(l); memcpy(n, d, l); return n;}

/* strip_spaces removes all leading and trailing spaces from the string, modifying it */
extern char *strip_spaces(char *);
/* strip_control removes all control characters (value < 32) from the string */
extern char *strip_control(char *);

extern size_t strlcpy(char *, const char *, size_t);

extern int xstrverscmp(const char *, const char *);
extern int xasprintf(char **, const char *, ...);

#ifdef strdup
# undef strdup
#endif
#ifdef strncpy
# undef strncpy
#endif
#ifdef strndup
# undef strndup
#endif
#pragma GCC poison strdup strncpy strndup asprintf

/** @}
 */

#endif
