/************************************************************************
 *   This is part of Dancer, a text relay daemon
 *   Copyright (C) 2002 Andrew Suffield <asuffield@users.sourceforge.net>
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 */

#ifdef MODULES_INTERNAL_H
struct module;
#else
#define MODULES_INTERNAL_H
#include <dancerd/config.h>

#include <dancerd/modules.h>

/** \file modules_internal.h
 * \brief Module system guts
 */

/** \defgroup moduleguts Module system guts
 * \ingroup mainguts
 *
 * @{
 */

struct module_handle;

/** \brief Loaded module */
struct module
{
  /** Name of the module */
  char *name;
  /** Version of the module currently loaded */
  char *version;
  /** Handle on the module object */
  struct module_handle *handle;
  /** True if the module cannot be unloaded */
  int permanent;
  struct llist *exported_symbols, *imported_symbols;
  struct llist *unload_handlers, *unload_handler_interests;
  struct llist *dependants;
  const struct module_dependency *dependencies;
};

struct module_dependant
{
  struct module *from;
  const char *version;
};

extern void unimport_all_symbols(struct module *);
extern void unexport_all_symbols(struct module *);
extern int export_symbol(struct module *, const char *, const char *, const char *, void *);
extern int unexport_symbol(struct module *, const char *, const char *, const char *);
extern int import_symbol(struct module *, const char *, const char *, const char *, void **, int);
extern void unimport_symbol(struct module *, const char *, const char *, const char *);
extern int check_busy_symbols(struct module *, int, const struct pending_symbol **);

extern struct module_handle *load_module_object(const char *, const char *);
extern void *get_object_symbol(struct module_handle *, const char *);
extern void unload_module_object(struct module_handle *);
extern const char *get_loader_error(void);
extern int process_symbol_table(struct module *, struct module *, const struct pending_symbol **);

/** @}
 */

#endif
