/************************************************************************
 *   This is part of Dancer, a text relay daemon
 *   Copyright (C) 2002 Andrew Suffield <asuffield@users.sourceforge.net>
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 */

#include <dancerd/config.h>
#include <dancerd/paths.h>
#include <dancerd/xmalloc.h>

#include "paths.tab"

#include <stdlib.h>
#include <string.h>

const char *pidfile = LOCALSTATEDIR "/run/dancerd/dancerd.pid";
const char *logfile = LOCALSTATEDIR "/log/dancerd/error.log";
const char *module_dir = PKGLIBDIR;

const char *logdir = LOCALSTATEDIR "/log/dancerd/";
const char *statedir = LOCALSTATEDIR "/lib/dancerd/";
const char *datadir = DATADIR "/dancerd/";
const char *localedir = DATADIR "/locale/";
const char *rundir = LOCALSTATEDIR "/run/dancerd/";
const char *confdir = SYSCONFDIR "/dancerd/";

static void
set_one_root(const char *new_root, const char **target)
{
  char *n;
  if (strncmp(*target, PREFIX, strlen(PREFIX)) == 0)
    *target += strlen(PREFIX);
  if (((*target)[0] == '/' ) && (new_root[strlen(new_root) - 1] == '/'))
    (*target)++;
  n = xmalloc(strlen(new_root) + strlen(*target) + 1);
  strcpy(n, new_root);
  strcat(n, *target);
  *target = n;
}

void
set_root(const char *new_root)
{
  static int called = 0;
  /* This function may be called zero or one times only */
  if (called)
    abort();
  called = 1;
  if (!new_root)
    return;
  set_one_root(new_root, &pidfile);
  set_one_root(new_root, &logdir);
  set_one_root(new_root, &statedir);
  set_one_root(new_root, &datadir);
  set_one_root(new_root, &localedir);
  set_one_root(new_root, &rundir);
  set_one_root(new_root, &module_dir);
  set_one_root(new_root, &logfile);
  set_one_root(new_root, &confdir);
}
