/************************************************************************
 *   This is part of Dancer, a text relay daemon
 *   Copyright (C) 2002 Andrew Suffield <asuffield@users.sourceforge.net>
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 */

#include <dancerd/config.h>
#include <dancerd/timer.h>
#include <dancerd/llist.h>
#include <dancerd/xmalloc.h>
#include <dancerd/clock.h>
#include <dancerd/ctor.h>

#include <string.h>

static time_t next_event = 0;
static struct time_hires next_event_hires = {.sec = 0, .usec = 0};

static struct llist *timers = NULL, *hires_timers = NULL;

struct timer
{
  time_t t;
  timer_handler_func *handler;
  void *data;
};

struct hires_timer
{
  struct time_hires t;
  timer_handler_func *handler;
  void *data;
};

void
poll_timer(void)
{
  struct time_hires t;
  if (next_event && (next_event < get_real_time()))
    {
      /* Handle event */
    }
  get_real_time_hires(&t);
  if ((t.sec > next_event_hires.sec)
      || ((t.sec == next_event_hires.sec) && t.usec > next_event_hires.usec))
    {
      /* Handle event */
    }
}

void
register_timer(time_t t, timer_handler_func *handler, void *data)
{
  struct timer *timer;
  struct llist_node *n;
  if (!handler)
    return;

  if (!timers)
    timers = new_llist();

  timer = xmalloc(sizeof(struct timer));
  timer->t = t;
  timer->handler = handler;
  timer->data = data;

  /* Find the first timer which will fire after the new event */
  for (n = llist_head(timers); n; n = llist_node_next(n))
    {
      struct timer *this_timer = llist_node_data(n);
      if (t < this_timer->t)
	break;
    }
  if (n)
    {
      insert_llist_before(n, timer);
      if (t < next_event)
	next_event = t;
    }
  else
    {
      add_llist_head(timers, timer);
      next_event = t;
    }
}

void
register_timer_hires(struct time_hires *t, timer_handler_func *handler, void *data)
{
  struct hires_timer *timer;
  struct llist_node *n;
  if (!handler)
    return;

  if (!hires_timers)
    hires_timers = new_llist();

  timer = xmalloc(sizeof(struct hires_timer));
  memcpy(&timer->t, t, sizeof(struct time_hires));
  timer->handler = handler;
  timer->data = data;

  /* Find the first timer which will fire after the new event */
  for (n = llist_head(hires_timers); n; n = llist_node_next(n))
    {
      struct hires_timer *this_timer = llist_node_data(n);
      if ((t->sec > this_timer->t.sec)
          || ((t->sec == this_timer->t.sec) && t->usec > this_timer->t.usec))
	break;
    }
  if (n)
    {
      insert_llist_before(n, timer);
      if ((t->sec > next_event_hires.sec)
          || ((t->sec == next_event_hires.sec) && t->usec > next_event_hires.usec))
	memcpy(&next_event_hires, t, sizeof(struct time_hires));
    }
  else
    {
      add_llist_head(hires_timers, timer);
      memcpy(&next_event_hires, t, sizeof(struct time_hires));
    }
}
