/************************************************************************
 *   This is part of Dancer, a text relay daemon
 *   Copyright (C) 2002 Andrew Suffield <asuffield@users.sourceforge.net>
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 */

#define DANCER_CORE__XMALLOC_SOURCE__

#include <dancerd/config.h>
#include <dancerd/xmalloc.h>
#include <dancerd/log.h>
#include <dancerd/restart.h>

#include <sys/types.h>
#include <stdlib.h>

void
out_of_memory(void)
{
  static int already_out_of_memory = 0; 
  if (already_out_of_memory)
    abort();
  already_out_of_memory = 1;
  log("core", LOG_PRIORITY_CRITICAL, "Out of memory. Attempting to restart.");
  restart();
}

void *
xmalloc(size_t len)
{
  void *data;
  data = malloc(len);
  if (!data)
    out_of_memory();
  return data;
}

void *
xcalloc(size_t len)
{
  void *data;
  data = malloc(len);
  if (!data)
    out_of_memory();
  memset(data, 0, len);
  return data;
}

void *
xrealloc(void *data, size_t len)
{
  data = realloc(data, len);
  if (!data && len)
    out_of_memory();
  return data;
}
