/************************************************************************
 *   This is part of Dancer, a text relay daemon
 *   Copyright (C) 2002 Andrew Suffield <asuffield@users.sourceforge.net>
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 */

#ifdef AUTH_INTERNAL_H
struct auth_handle;
#else
#define AUTH_INTERNAL_H
#include <dancerd/config.h>
#include <dancerd/modules.h>
#include <dancerd/auth/auth.h>
#include <dancerd/llist.h>

/** \file auth_internal.h
 * \brief Internals of the authentication layer interface
 */

/** \defgroup authguts Internals of the authentication layer interface
 * \ingroup auth
 * This group collects the stuff used internally by the authentication
 * layer interface (the auth module itself).
 *
 * @{
 */ 

/** \brief A handle for a registered authentication method */
struct auth_method
{
  /** Name of this method */
  char *name;
  /** List of all the handles using this method at present */
  struct llist *handles;
  /** Module that contains this method */
  struct module *module;
  /** Function to be called to initialise the method on a new handle */
  auth_init_handle *aih;
  /** Called when a handle is cancelled */
  auth_cancel_handle *ach;
  /** Called on peer_accept() */
  auth_peer_accept *apa;
  /** Called on read_data() */
  auth_read_data *ard;
  /** Called on get_data() */
  auth_get_data *agd;
  /** Called on set_credentials() */
  auth_set_credentials *asc;
};

/** \brief A handle for a current authentication session */
struct auth_handle
{
  /** Name of the entity this session is authenticating against */
  char *entity;
  /** Method this handle is using */
  struct auth_method *method;
  /** Data handle used by the method */
  void *data;
  /** Node this handle occupies in module->handles */
  struct llist_node *handles_node;
};

/** @}
 */

#endif
