/************************************************************************
 *   This is part of Dancer, a text relay daemon
 *   Copyright (C) 2002 Andrew Suffield <asuffield@users.sourceforge.net>
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 */

#ifdef CONFIG_ITEMS_H

struct config_item;

#else
#define CONFIG_ITEMS_H
#include <dancerd/config.h>
#include <dancerd/modules.h>

#include <stdint.h>

/** \file config/items.h
 * \brief Configuration items
 */

/** \defgroup configitem Configuration items
 *
 * \par
 * A configuration item is a representation of a key/value pair that
 * is stored across invocations of the daemon, and may be writeable.
 *
 * The key is a string; the value is a boolean, a string, or a 32-bit
 * integer (signed or unsigned).
 *
 * @{
 */

/** \brief Configuration item
 *
 * A config item is stored (somehow, by a config module) in a
 * permanent manner, lasting across different instances of the daemon
 * and synchronised between them when updated.
 *
 * This struct is a copy of the item, retrieved from storage by
 * config::get_item and stored there by config::put_item.
 */
struct config_item
{
  char *key;
  /* Only one of these values can be used */
  enum
    {
      citem_null = 0, citem_bool, citem_string, citem_uint32, citem_sint32
    } type;
  /* There is no valid justification for using a union here */
  char *str;
  uint32_t uint32;
  int32_t sint32;
  char boolean;
};

DEFINE_NAMESPACE(config, 0);

/* get_item() returns a citem_null if the target item is not found */
DECLARE_SYMBOL(int, config, get_item, (char *, struct config_item *));
DECLARE_SYMBOL(int, config, put_item, (struct config_item *));
DECLARE_SYMBOL(int, config, delete_item, (char *));

END_NAMESPACE(config);

/**
 * @}
 */

#endif
