/************************************************************************
 *   This is part of Dancer, a text relay daemon
 *   Copyright (C) 2002 Andrew Suffield <asuffield@users.sourceforge.net>
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 */

#ifdef DATABASE_DATABASE_H

struct database;

#else
#define DATABASE_DATABASE_H
#include <dancerd/config.h>
#include <dancerd/modules.h>

#include <stdint.h>

/** \file database/database.h
 * Database abstraction layer
 */

/** \defgroup database Database abstraction layer
 *
 * \par
 * This is a generic interface for simple access to whatever backend
 * database is currently in use.
 *
 * @{
 */

struct database;
struct database_cursor;
struct database_transaction;

/* Returns 1 for index, 0 for don't index, anything else for error */
typedef int associated_key_callback(void *, uint32_t, void *, uint32_t, void **, uint32_t *);
#define ASSOCIATED_KEY_STORE 1
#define ASSOCIATED_KEY_IGNORE 0
#define ASSOCIATED_KEY_STORE_MALLOCED 2

DEFINE_NAMESPACE(database, 0);

DECLARE_SYMBOL(const char *, database, driver_name, (void)) pure_attribute;
DECLARE_SYMBOL(struct database *, database, open_database, (const char *, const char *,
                                                            struct database_transaction *,
                                                            int dup));
DECLARE_SYMBOL(void, database, close_database, (struct database *));
DECLARE_SYMBOL(int, database, associate_database, (struct database *,
                                                   struct database *,
                                                   struct database_transaction *,
                                                   associated_key_callback *));
/* get_value and delete_value return 1 on success, 0 on not-found, -1 on failure */
DECLARE_SYMBOL(int, database, get_value, (struct database *,
                                          struct database_transaction *,
                                          void *, uint32_t,
                                          void **, uint32_t *));
/* put_value returns 1 on success and -1 on failure */
DECLARE_SYMBOL(int, database, put_value, (struct database *,
                                          struct database_transaction *,
                                          void *, uint32_t,
                                          void *, uint32_t));
DECLARE_SYMBOL(int, database, delete_value, (struct database *,
                                             struct database_transaction *,
                                             void *, uint32_t));
DECLARE_SYMBOL(int, database, check_sanity, (struct database *));

DECLARE_SYMBOL(struct database_cursor *, database, get_cursor, (struct database *,
                                                                struct database_transaction *,
                                                                void *, uint32_t,
                                                                void **, uint32_t *));
DECLARE_SYMBOL(struct database_cursor *, database, get_primary_cursor, (struct database *,
                                                                        struct database_transaction *,
                                                                        void *, uint32_t,
                                                                        void **, uint32_t *));
DECLARE_SYMBOL(void, database, free_cursor, (struct database_cursor *));
DECLARE_SYMBOL(int, database, cursor_next, (struct database_cursor *, void **, uint32_t *));
DECLARE_SYMBOL(int, database, cursor_primary_next, (struct database_cursor *, void **, uint32_t *));
DECLARE_SYMBOL(int, database, cursor_prev, (struct database_cursor *, void **, uint32_t *));
DECLARE_SYMBOL(int, database, cursor_primary_prev, (struct database_cursor *, void **, uint32_t *));
DECLARE_SYMBOL(int, database, cursor_get, (struct database_cursor *, void **, uint32_t *));
DECLARE_SYMBOL(int, database, cursor_primary_get, (struct database_cursor *, void **, uint32_t *));
DECLARE_SYMBOL(int, database, cursor_next_dup, (struct database_cursor *, void **, uint32_t *));
DECLARE_SYMBOL(int, database, cursor_primary_next_dup, (struct database_cursor *, void **, uint32_t *));
DECLARE_SYMBOL(int, database, cursor_delete, (struct database_cursor *));

DECLARE_SYMBOL(struct database_cursor *, database, join, (struct database *,
                                                          struct database_transaction *,
                                                          int,
                                                          void **, uint32_t *));

DECLARE_SYMBOL(struct database_transaction *, database, begin_transaction, (void));
DECLARE_SYMBOL(int, database, commit_transaction, (struct database_transaction *));
DECLARE_SYMBOL(void, database, abort_transaction, (struct database_transaction *));

DECLARE_SYMBOL(int, database, run_transaction, (int (*)(struct database_transaction *, void *), void *, int));

END_NAMESPACE(database);

/**
 * @}
 */

#endif
