/************************************************************************
 *   This is part of Dancer, a text relay daemon
 *   Copyright (C) 2002 Andrew Suffield <asuffield@users.sourceforge.net>
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 */

#ifdef RFC1459_CLIENT_CHARSET_H
struct rfc1459_charset;
#else
#define RFC1459_CLIENT_CHARSET_H

#include <dancerd/llist.h>
#include <iconv.h>
#include <sys/types.h>

#define RFC1459_CLIENT_VERSION "1.0"

struct rfc1459_charset
{
  char *name;
  iconv_t cd_to_utf8;
  iconv_t cd_from_utf8;
  uint32_t refs;
  struct llist_node *node;
};

extern struct rfc1459_charset *new_iconv(const char *);
extern void release_iconv(struct rfc1459_charset *);
extern char *convert_to_utf8(struct rfc1459_charset *, const char *);
extern char *convert_from_utf8(struct rfc1459_charset *, const char *);
extern void init_charset(void);
extern void reinit_charset(void);
extern void deinit_charset(int);

#endif
