/************************************************************************
 *   This is part of Dancer, a text relay daemon
 *   Copyright (C) 2002 Andrew Suffield <asuffield@users.sourceforge.net>
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 */

#ifdef RFC1459_CLIENT_H
#else
#define RFC1459_CLIENT_H

#include <dancerd/config.h>
#include <dancerd/hashtable.h>
#include <gettext.h>
#include <iconv.h>

#define RFC1459_CLIENT_VERSION "1.0"

struct rfc1459_charset;

struct rfc1459_client
{
  struct session *session;
  char *name;
  char *myname;
  int done_user;
  int done_nick;
  int done_handshaking;
  struct rfc1459_charset *charset;
  char *language;
};

struct rfc1459_command;

typedef void parse_command_func(struct rfc1459_client *, const struct rfc1459_command *, unsigned int, char **);

struct rfc1459_command
{
  const char *name;
  unsigned int min_params;
  unsigned int max_params;
  parse_command_func *handler;
};

/* Mirror the logic in gettext.h here; if ENABLE_NLS is not defined or
 * false, we inline the strings
 */
#if ENABLE_NLS
extern const char *rfc1459_client_gettext(struct rfc1459_client *, const char *);
extern const char *rfc1459_client_ngettext(struct rfc1459_client *, const char *, const char *, unsigned long int);

#define _c(C, S) rfc1459_client_gettext(C, S)
#define _cn(C, S, SP, N) rfc1459_client_ngettext(C, S, SP, N)
#else
#define _c(C, S) ((const char *)(C))
#define _cn(C, S, SP, N) ((N) == 1 ? (const char *) (S) : (const char *) (SP))
#endif

#endif
