/************************************************************************
 *   This is part of Dancer, a text relay daemon
 *   Copyright (C) 2002 Andrew Suffield <asuffield@users.sourceforge.net>
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 */

#ifdef RFC1459_MESSAGES_H
#else
#define RFC1459_MESSAGES_H

#include <dancerd/config.h>
#include "client.h"

typedef void message_send_handler(struct rfc1459_client *, const struct message *);

struct rfc1459_message
{
  const char *message_type_name;
  int message_type;
  message_send_handler *handler;
  /** Bitfield of required message components */
  uint16_t required_components;
  /** Minimum number of string components required */
  int min_strc;
  /** Minimum number of uint32 components required */
  int min_uint32c;
  /** Minimum number of sint32 components required */
  int min_sint32c;
};

#define RMC_ORIGIN 0x0001
#define RMC_DEST   0x0002

extern struct rfc1459_message *rfc1459_messages;
extern int max_rfc1459_message_type;
extern struct rfc1459_message **rfc1459_message_types;

extern void init_rfc1459_messages(void);

#endif
