/************************************************************************
 *   This is part of Dancer, a text relay daemon
 *   Copyright (C) 2002 Andrew Suffield <asuffield@users.sourceforge.net>
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 */

#ifdef SCHEDULER_H

struct scheduler;

#else
#define SCHEDULER_H
#include <dancerd/config.h>
#include <dancerd/modules.h>
#include <dancerd/protocol/message.h>

#include <stdint.h>

/** \file scheduler.h
 * \brief Scheduling for processing of messages
 */

/** \defgroup scheduler Message schedulers
 * \ingroup protocol
 * A scheduler decides when and in what order messages will be processed.
 *
 * @{
 */

struct scheduler;

typedef void add_session_handler(struct session *);
typedef void remove_session_handler(struct session *);
typedef void queue_packet_handler(struct session *, struct message *);

DEFINE_NAMESPACE(scheduler, 0);

DECLARE_SYMBOL(struct scheduler *, scheduler, register_scheduler, (const char *, queue_packet_handler *,
                                                                    add_session_handler *, remove_session_handler *));
DECLARE_SYMBOL(void, scheduler, unregister_scheduler, (struct scheduler *));
DECLARE_SYMBOL(struct scheduler *, scheduler, get_scheduler, (const char *));

DECLARE_SYMBOL(void, scheduler, add_session, (struct session *, struct scheduler *));
DECLARE_SYMBOL(void, scheduler, remove_session, (struct session *, struct scheduler *));

DECLARE_SYMBOL(void, scheduler, queue_message, (struct session *, struct scheduler *, struct message *));

END_NAMESPACE(scheduler);

/** @}
 */

#endif
