/************************************************************************
 *   This is part of Dancer, a text relay daemon
 *   Copyright (C) 2002 Andrew Suffield <asuffield@users.sourceforge.net>
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 */

#include <dancerd/config.h>
#include <dancerd/xmalloc.h>

#include "node.h"
#include "handlers.h"
#include "send.h"
#include "router.h"

void
m_close_session(struct peer *peer unused_attribute, struct node *origin, const struct message *msg)
{
  if (msg->strv[0])
    {
      struct node *target = hashtable_lookup(node_hash, msg->strv[0]);
      /* If the target doesn't exist, forget it */
      if (!target)
        return;
      /* If the target is me, then flip this around - close my session to
       * origin (ie, a local node is closing its session now)
       */
      if (target == me)
        target = origin;
      /* If the target is not attached to that peer, the peer is
       * lying. Kill it.
       */
      if (target->peer != peer)
        {
          do_close_session(NULL, peer->node, "Invalid target in 'close session' message", 0);
          return;
        }
      do_close_session(origin, target, msg->strc >= 2 ? msg->strv[1] : NULL, 0);
    }
  else
    {
      /* If there is no target, then the peer is the target - this can
       * happen before the client name has been agreed, for example
       */
      do_close_session(origin, peer->node, msg->strc >= 2 ? msg->strv[1] : NULL, 0);
    }
}
