/************************************************************************
 *   This is part of Dancer, a text relay daemon
 *   Copyright (C) 2002 Andrew Suffield <asuffield@users.sourceforge.net>
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 */

#ifdef ROUTER_HANDLERS_H
#else
#define ROUTER_HANDLERS_H

#include <dancerd/config.h>
#include <dancerd/protocol/message.h>

#include "node.h"

typedef void router_message_handler_func(struct peer *, struct node *, const struct message *);
typedef int router_message_filter_func(struct peer *, struct node *, struct node *, const struct message *);

extern router_message_handler_func m_set_name;
extern router_message_handler_func m_set_real_name;
extern router_message_handler_func m_query_node_information;
extern router_message_handler_func m_query_node_software_information;
extern router_message_handler_func m_query_node_license_information;
extern router_message_handler_func m_query_node_author_information;
extern router_message_handler_func m_done_handshake;
extern router_message_handler_func m_accept_handshake;
extern router_message_handler_func m_close_session;
extern router_message_handler_func m_private;
extern router_message_filter_func f_private;

extern router_message_handler_func m_reject_name;
extern router_message_handler_func m_reject_real_name;
extern router_message_handler_func m_invalid_destination;
extern router_message_filter_func error_filter;

extern router_message_handler_func m_echo_request;
extern router_message_handler_func m_echo_reply;

extern router_message_handler_func m_begin_auth;
extern router_message_handler_func m_cancel_auth;
extern router_message_handler_func m_accept_auth;
extern router_message_handler_func m_auth_data;
extern router_message_handler_func m_set_credentials;
extern router_message_handler_func m_register_user;

extern router_message_handler_func m_begin_connection;
extern router_message_handler_func m_query_routable_nodes;
extern router_message_handler_func m_done_routable_nodes;
extern router_message_handler_func m_routable_node;

#define PARAM_PAIR(X) int min_ ## X ## c; int max_ ## X ## c
struct message_handler
{
  const char *type_name;
  int type;
  router_message_handler_func *handler;
  router_message_filter_func *filter;
  uint16_t required;
  PARAM_PAIR(str);
  PARAM_PAIR(uint32);
  PARAM_PAIR(sint32);
};
#undef PARAM_PAIR

/* Requre message components "origin" or "dest" */
#define RMC_ORIGIN    0x0001
#define RMC_DEST      0x0002
/* Require origin node to be authenticated */
#define RON_AUTH      0x0004
/* Allow origin node to be handshaking (ie, they don't have to have completed) */
#define AON_HANDSHAKE 0x0008
/* Require origin node to be a router */
#define RON_ROUTER    0x0010

extern struct message_handler *message_handlers;

extern int router_attribute;

extern new_session_handler new_session;

extern void init_message_handlers(void);
extern void deinit_message_handlers(void);

#endif
