/************************************************************************
 *   Copyright (C) 2003 Andrew Suffield <asuffield@users.sourceforge.net>
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 */

#include <dancerd/config.h>
#include <dancerd/xmalloc.h>
#include <dancerd/xstring.h>

#include <assert.h>

#include "router.h"
#include "node.h"
#include "handlers.h"
#include "send.h"

EXTERN_MODULE();

#include "node.c.imptab"

struct hashtable *node_hash;

struct node *
make_node(struct peer *peer)
{
  struct node *node = xmalloc(sizeof(*node));
  memset(node, 0, sizeof(*node));

  node->name = NULL;
  node->authenticated = false;
  node->next_authenticated_name = NULL;
  node->next_authenticated = false;
  node->realname = NULL;
  node->language = NULL;
  node->auth = NULL;
  node->routers_entry = NULL;
  node->adjacent_peers_entry = NULL;
  node->adjacent = NULL;

  node->peer = peer;
  node->peers = new_llist();

  node->router = false;
  node->is_closing = false;

  add_llist_head(global_nodes, node);
  node->global_nodes_entry = llist_head(global_nodes);

  return node;
}

struct peer *
make_peer(struct session *session)
{
  struct peer *peer = xmalloc(sizeof(*peer));
  memset(peer, 0, sizeof(*peer));

  peer->node = NULL;
  peer->me = me;
  peer->session = session;
  peer->accepted_my_handshake = false;
  peer->accepted_peer_handshake = false;
  peer->sent_my_handshake = false;

  SYM(protocol, set_attribute)(session, router_attribute, peer);

  return peer;
}

void
free_peer(struct peer *peer)
{
  xfree(peer);
}

void
free_node(struct node *node)
{
  delete_llist_node(node->global_nodes_entry);
  delete_llist_node(node->routers_entry);
  delete_llist_node(node->adjacent_peers_entry);

  free_llist(node->peers);

  xfree(node->realname);
  xfree(node->name);
  if (node->peer && node->peer->node == node)
    free_peer(node->peer);
  xfree(node);
}
