/************************************************************************
 *   This is part of Dancer, a text relay daemon
 *   Copyright (C) 2002 Andrew Suffield <asuffield@users.sourceforge.net>
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 */

#define _GNU_SOURCE
#include <dancerd/config.h>
#include <dancerd/xmalloc.h>
#include <dancerd/version.h>

#include <stdio.h>
#include <assert.h>

#include "node.h"
#include "handlers.h"
#include "send.h"

void
m_query_node_information(struct peer *peer unused_attribute,
                         struct node *origin, const struct message *msg unused_attribute)
{
  struct node *node = hashtable_lookup(node_hash, msg->strv[0]);

  if (!node)
    {
      send_failed_node_information(origin, msg->strv[0], NULL);
      return;
    }

  send_node_information(origin, node->name, NULL, node->realname);
}

void
m_query_node_software_information(struct peer *peer unused_attribute,
                                  struct node *origin, const struct message *msg unused_attribute)
{
  char *build_info;
  asprintf(&build_info, "Built at %s, on %s", build_time, build_platform);
  send_node_software_information(origin, "dancerd", dancer_version, serial_number,
                                 build_info, _n(origin, "Dancer: a text relay daemon"));
  xfree(build_info);
}

void
m_query_node_license_information(struct peer *peer unused_attribute,
                                   struct node *origin, const struct message *msg unused_attribute)
{
  send_node_license_information(origin, license_lines, license);
}

void
m_query_node_author_information(struct peer *peer unused_attribute,
                                struct node *origin, const struct message *msg unused_attribute)
{
  send_node_author_information(origin, authors_lines, authors);
}
