/************************************************************************
 *   This is part of Dancer, a text relay daemon
 *   Copyright (C) 2002 Andrew Suffield <asuffield@users.sourceforge.net>
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 */

#ifdef ROUTER_SEND_H
#else
#define ROUTER_SEND_H

#include <dancerd/config.h>
#include <dancerd/protocol/message.h>

#include "node.h"

extern void send_wrong_parameters(struct node *, int, const char *);
extern void send_handshake_required(struct node *, int);
extern void send_invalid_destination(struct node *, const char *, int);
extern void send_accept_handshake(struct node *);
extern void send_done_handshake(struct node *);
extern void send_close_session(struct node *, struct node *, struct node *, const char *);
extern void send_reject_set_name(struct node *, const char *, const char *);
extern void send_set_name(struct node *, const char *, const char *);
extern void send_set_real_name(struct node *, struct node *, const char *);
extern void send_fatal_error(struct node *, const char *);
extern void send_relaying_denied(struct node *, struct node *);
extern void send_node_information(struct node *, const char *, const char *, const char *);
extern void send_failed_node_information(struct node *, const char *, const char *);
extern void send_node_software_information(struct node *,
                                           const char *, const char *, const char *, const char *, const char *);
extern void send_node_license_information(struct node *, int, const char **);
extern void send_node_author_information(struct node *, int, const char **);
extern void send_accept_auth(struct node *);
extern void send_cancel_auth(struct node *, const char *);
extern void send_auth_data(struct node *, int, char **);
extern void send_not_authenticated(struct node *, uint32_t);
extern void send_not_authorised(struct node *, uint32_t);
extern void send_accept_set_credentials(struct node *, const char *);
extern void send_reject_set_credentials(struct node *, const char *, const char *);

extern void send_accept_register_user(struct node *);
extern void send_reject_register_user(struct node *, const char *);
extern void send_unregistered_user(struct node *);
extern void send_registered_user(struct node *);
extern void send_node_authenticated(struct node *, struct node *);
extern void send_node_not_authenticated(struct node *, struct node *);

extern void send_query_routable_nodes(struct node *);
extern void send_done_routable_nodes(struct node *);
extern void send_routable_node(struct node *, struct node *);

extern void send_echo_request(struct node *, const char *);
extern void send_echo_reply(struct node *, const char *);

extern void init_send(void);

#endif
