/************************************************************************
 *   This is part of Dancer, a text relay daemon
 *   Copyright (C) 2002 Andrew Suffield <asuffield@users.sourceforge.net>
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 */

#ifdef TCLASS_H

struct transport_class;

#else
#define TCLASS_H
#include <dancerd/config.h>
#include <dancerd/modules.h>
#include <dancerd/llist.h>

#include "transport_internal.h"

/** \file tclass.h
 * \brief Transport classes
 */

/** \defgroup transportclass Transport classes
 * \ingroup transportguts
 * Some support for handling transport classes
 *
 * @{
 */ 

/** \brief A transport class
 */
struct transport_class
{
  /** Name of the class */
  char *name;
  /** Transport used by this class for new connections */
  struct transport *trans;
  /** Arbitrary handle passed to the transport */
  void *tdata;
  /** List of connections using this class */
  struct llist *connections;
  /** Node that contains this class in trans->classes */
  struct llist_node *transport_classes_node;
  /** True if this class is currently active (accepting connections) */
  int active;
};

extern void read_transport_class(const char *);
extern void check_transport_class(struct transport_class *);
extern struct transport_class *find_transport_class(const char *);

extern struct llist *classes;

/** @}
 */

#endif
