# Copyright (C) 2002 Andrew Suffield <asuffield@users.sourceforge.net>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA

use strict;
use 5.006;
use warnings;
use DejaPerl qw/:framework/;
use Dancer::CmdPipe;
use Dancer::Spawn;

my $dancer = new Dancer::Spawn "config";
$dancer->install();
$dancer->start(modules => [qw/basic.logfile database.db config.database test.config/]);
my $pipe = new Dancer::CmdPipe $dancer, "config", "test.config", sanity_checks => 0;

verbose "Starting test run...";

# Booleans
$pipe->do("PUT test.config.bool.alpha t", "put config value");
$pipe->check_cmd_value("GET test.config.bool.alpha", "true", "get config value");
$pipe->do("PUT test.config.bool.beta t", "put config value");
$pipe->check_cmd_value("GET test.config.bool.beta", "true", "get config value");
$pipe->do("PUT test.config.bool.gamma f", "put config value");
$pipe->check_cmd_value("GET test.config.bool.alpha", "true", "get config value");
$pipe->check_cmd_value("GET test.config.bool.beta", "true", "get config value");
$pipe->check_cmd_value("GET test.config.bool.gamma", "false", "get config value");
# Update a value
$pipe->do("PUT test.config.bool.alpha f", "put config value");
$pipe->check_cmd_value("GET test.config.bool.alpha", "false", "get config value");
# A non-existant value
$pipe->check_cmd_value("GET test.config.bool.delta", "null", "get config value");
# Delete a value
$pipe->do("DELETE test.config.bool.beta", "delete config value");
$pipe->check_cmd_value("GET test.config.bool.beta", "null", "get config value");

# Strings
$pipe->do("PUT test.config.string.alpha \"foo\"", "put config value");
$pipe->do("PUT test.config.string.beta \"bar\"", "put config value");
$pipe->do("PUT test.config.string.gamma \"baz\"", "put config value");
$pipe->check_cmd_value("GET test.config.string.alpha", "\"foo\"", "get config value");
$pipe->check_cmd_value("GET test.config.string.beta", "\"bar\"", "get config value");
$pipe->check_cmd_value("GET test.config.string.gamma", "\"baz\"", "get config value");
$pipe->do("PUT test.config.string.beta \"quux\"", "put config value");
$pipe->check_cmd_value("GET test.config.string.alpha", "\"foo\"", "get config value");
$pipe->check_cmd_value("GET test.config.string.beta", "\"quux\"", "get config value");
$pipe->check_cmd_value("GET test.config.string.gamma", "\"baz\"", "get config value");
$pipe->do("DELETE test.config.string.gamma", "delete config value");
$pipe->check_cmd_value("GET test.config.string.alpha", "\"foo\"", "get config value");
$pipe->check_cmd_value("GET test.config.string.beta", "\"quux\"", "get config value");
$pipe->check_cmd_value("GET test.config.string.gamma", "null", "get config value");

# Unsigned ints
$pipe->do("PUT test.config.uint.alpha 12", "put config value");
$pipe->do("PUT test.config.uint.beta 17", "put config value");
$pipe->do("PUT test.config.uint.gamma 432", "put config value");
$pipe->check_cmd_value("GET test.config.uint.alpha", "12", "get config value");
$pipe->check_cmd_value("GET test.config.uint.beta", "17", "get config value");
$pipe->check_cmd_value("GET test.config.uint.gamma", "432", "get config value");
$pipe->do("PUT test.config.uint.gamma 234", "put config value");
$pipe->check_cmd_value("GET test.config.uint.alpha", "12", "get config value");
$pipe->check_cmd_value("GET test.config.uint.beta", "17", "get config value");
$pipe->check_cmd_value("GET test.config.uint.gamma", "234", "get config value");
$pipe->do("DELETE test.config.uint.gamma", "delete config value");
$pipe->check_cmd_value("GET test.config.uint.alpha", "12", "get config value");
$pipe->check_cmd_value("GET test.config.uint.beta", "17", "get config value");
$pipe->check_cmd_value("GET test.config.uint.gamma", "null", "get config value");

# Signed ints
$pipe->do("PUT test.config.sint.alpha +12", "put config value");
$pipe->do("PUT test.config.sint.beta -17", "put config value");
$pipe->do("PUT test.config.sint.gamma -432", "put config value");
$pipe->check_cmd_value("GET test.config.sint.alpha", "+12", "get config value");
$pipe->check_cmd_value("GET test.config.sint.beta", "-17", "get config value");
$pipe->check_cmd_value("GET test.config.sint.gamma", "-432", "get config value");
$pipe->do("PUT test.config.sint.gamma +234", "put config value");
$pipe->check_cmd_value("GET test.config.sint.alpha", "+12", "get config value");
$pipe->check_cmd_value("GET test.config.sint.beta", "-17", "get config value");
$pipe->check_cmd_value("GET test.config.sint.gamma", "+234", "get config value");

$pipe->do("DELETE test.config.sint.gamma", "delete config value");
$pipe->check_cmd_value("GET test.config.sint.alpha", "+12", "get config value");
$pipe->check_cmd_value("GET test.config.sint.beta", "-17", "get config value");
$pipe->check_cmd_value("GET test.config.sint.gamma", "null", "get config value");

$dancer->kill();

# Now restart it and check the values persisted

$dancer->start(modules => [qw/database.db config.database test.config/]);
$pipe = new Dancer::CmdPipe $dancer, "config", "test.config", sanity_checks => 0;

$pipe->check_cmd_value("GET test.config.bool.alpha", "false", "get config value");
$pipe->check_cmd_value("GET test.config.bool.beta", "null", "get config value");
$pipe->check_cmd_value("GET test.config.bool.gamma", "false", "get config value");
$pipe->check_cmd_value("GET test.config.string.alpha", "\"foo\"", "get config value");
$pipe->check_cmd_value("GET test.config.string.beta", "\"quux\"", "get config value");
$pipe->check_cmd_value("GET test.config.string.gamma", "null", "get config value");
$pipe->check_cmd_value("GET test.config.uint.alpha", "12", "get config value");
$pipe->check_cmd_value("GET test.config.uint.beta", "17", "get config value");
$pipe->check_cmd_value("GET test.config.uint.gamma", "null", "get config value");
$pipe->check_cmd_value("GET test.config.sint.alpha", "+12", "get config value");
$pipe->check_cmd_value("GET test.config.sint.beta", "-17", "get config value");
$pipe->check_cmd_value("GET test.config.sint.gamma", "null", "get config value");

verbose "Finished test run";

$dancer->kill();

1;
