# Copyright (C) 2002 Andrew Suffield <asuffield@users.sourceforge.net>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA

use strict;
use 5.006;
use warnings;
use DejaPerl qw/:framework/;
use Expect;

my $dancer = new Dancer::Spawn "edit-config.pl";
$dancer->install();
verbose "Beginning test run";

sub do_edit(@)
  {
    my $exp = new Expect;
    $exp->raw_pty(1);
    $exp->log_user($all_flag);
    $exp->log_file($log);
    $exp->spawn("$dancer/sbin/dancer-edit-config",
                "--dancerd=$dancer/sbin/dancerd",
                "--dancer-args=-d$dancer",
                "--", @_);
    return $exp;
  }

sub put_config($$$)
  {
    my ($name, $type, $value) = @_;
    my $exp = do_edit("put", "test.edit-config-pl.$name", $type, $value);
    check_expect $exp, qr/\QPut key 'test.edit-config-pl.$name' as $type $value\E\n/, "put $name", 8;
    $exp->soft_close();
  }

sub get_config($$$)
  {
    my ($name, $type, $value) = @_;
    my $exp = do_edit("get", "test.edit-config-pl.$name");
    $value = ($type eq 'null') ? "" : " $value";
    check_expect $exp, qr/\QKey 'test.edit-config-pl.$name' is $type$value\E\n/, "get $name", 8;
    $exp->soft_close();
  }

sub delete_config($)
  {
    my ($name) = @_;
    my $exp = do_edit("delete", "test.edit-config-pl.$name");
    check_expect $exp, qr/\QDeleted key 'test.edit-config-pl.$name'\E\n/, "delete $name", 8;
    $exp->soft_close();
  }

get_config "nonexistant", "null", "";
get_config "", "null", "";

put_config "bool.alpha", "boolean", "true";
put_config "bool.beta", "boolean", "true";
put_config "bool.gamma", "boolean", "false";
get_config "bool.alpha", "boolean", "true";
get_config "bool.beta", "boolean", "true";
get_config "bool.gamma", "boolean", "false";
put_config "bool.beta", "boolean", "false";
delete_config "bool.gamma";
get_config "bool.alpha", "boolean", "true";
get_config "bool.beta", "boolean", "false";
get_config "bool.gamma", "null", "";

put_config "string.alpha", "string", "foo";
put_config "string.beta", "string", "bar";
put_config "string.gamma", "string", "baz";
put_config "string.delta", "string", "";
get_config "string.alpha", "string", "foo";
get_config "string.beta", "string", "bar";
get_config "string.gamma", "string", "baz";
get_config "string.delta", "string", "";
put_config "string.alpha", "string", "quux";
delete_config "string.beta";
get_config "string.alpha", "string", "quux";
get_config "string.beta", "null", "";
get_config "string.gamma", "string", "baz";
get_config "string.delta", "string", "";

put_config "uint.alpha", "integer", "12";
put_config "uint.beta", "integer", "17";
put_config "uint.gamma", "integer", "774";
get_config "uint.alpha", "integer", "12";
get_config "uint.beta", "integer", "17";
get_config "uint.gamma", "integer", "774";
put_config "uint.gamma", "integer", "929";
delete_config "uint.alpha";
get_config "uint.alpha", "null", "";
get_config "uint.beta", "integer", "17";
get_config "uint.gamma", "integer", "929";

put_config "sint.alpha", "integer", "-12";
put_config "sint.beta", "integer", "+17";
put_config "sint.gamma", "integer", "-774";
get_config "sint.alpha", "integer", "-12";
get_config "sint.beta", "integer", "+17";
get_config "sint.gamma", "integer", "-774";
put_config "sint.gamma", "integer", "+924";
delete_config "sint.alpha";
get_config "sint.alpha", "null", "";
get_config "sint.beta", "integer", "+17";
get_config "sint.gamma", "integer", "+924";

verbose "Test run complete";

1;
