# Copyright (C) 2002 Andrew Suffield <asuffield@users.sourceforge.net>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA

use strict;
use 5.006;
use warnings;
use DejaPerl qw/:framework/;
use Dancer::Spawn;

my $dancer = new Dancer::Spawn "edit.config";
$dancer->install();
$dancer->start(args => [qw/-e/], modules => [qw/database.db config.database edit.config/]);

my $exp = $dancer->spawned();
check_expect $exp, qr/edit\.config ready\n/, "ready message" or exit 1;
my $timeout = 3;

sub put($$)
  {
    my $name = shift;
    my $value = shift;
    $exp->send("PUT test.edit.config.$name $value\n");
    my (undef, $err, undef, undef, undef) =
      $exp->expect($timeout,
                   "PUT test.edit.config.$name\n", sub {pass "put $name"},
                   -re => "PUT [^\n]*\n", sub {fail "put $name (wrong name)"},
                   -re => "ERROR: [^\n]*\n", sub {my $exp = shift; my $match = $exp->match; fail "put $name ($match)"},
                  );
    if ($err)
      {
        if ($err =~ /TIMEOUT/)
          {
            fail "put $name (timeout)";
          }
        elsif ($err =~ /EOF/)
          {
            error "Unexpected EOF from dancer";
            exit 1;
          }
        elsif ($err =~ s/^3://)
          {
            error $err;
          }
        else
          {
            error "Unknown error: $err (\$! eq $!)";
          }
      }
  }

sub get($$)
  {
    my $name = shift;
    my $value = shift;
    $exp->send("GET test.edit.config.$name\n");
    my (undef, $err, undef, undef, undef) =
      $exp->expect($timeout,
                   "GET test.edit.config.$name: $value\n", sub {pass "get $name"},
                   -re => "GET [^\n]*: $value\n", sub {fail "get $name (wrong name)"},
                   -re => "GET $name: [^\n]*\n", sub {fail "get $name (wrong value)"},
                   -re => "GET [^\n]*: [^\n]*\n", sub {fail "get $name (wrong name AND wrong value)"},
                   -re => "ERROR: [^\n]*\n", sub {my $exp = shift; my $match = $exp->match; fail "get $name ($match)"},
                  );
    if ($err)
      {
        if ($err =~ /TIMEOUT/)
          {
            fail "get $name (timeout)";
          }
        elsif ($err =~ /EOF/)
          {
            error "Unexpected EOF from dancer";
            exit 1;
          }
        elsif ($err =~ s/^3://)
          {
            error $err;
          }
        else
          {
            error "Unknown error: $err (\$! eq $!)";
          }
      }
  }

sub del($)
  {
    my $name = shift;
    $exp->send("DELETE test.edit.config.$name\n");
    my (undef, $err, undef, undef, undef) =
      $exp->expect($timeout,
                   "DELETE test.edit.config.$name: ok\n", sub {pass "delete $name"},
                   -re => "DELETE test.edit.config.$name: [^\n]*\n", sub {fail "delete $name (failed)"},
                   -re => "DELETE [^\n]*\n", sub {fail "delete $name (wrong name)"},
                   -re => "ERROR: [^\n]*\n", sub {my $exp = shift; my $match = $exp->match; fail "delete $name ($match)"},
                  );
    if ($err)
      {
        if ($err =~ /TIMEOUT/)
          {
            fail "delete $name (timeout)";
          }
        elsif ($err =~ /EOF/)
          {
            error "Unexpected EOF from dancer";
            exit 1;
          }
        elsif ($err =~ s/^3://)
          {
            error $err;
          }
        else
          {
            error "Unknown error: $err (\$! eq $!)";
          }
      }
  }

get "nonexistant", "null";
get "", "null";

put "bool.alpha", "true";
put "bool.beta", "true";
put "bool.gamma", "false";
get "bool.alpha", "true";
get "bool.beta", "true";
get "bool.gamma", "false";
put "bool.beta", "false";
del "bool.gamma";
get "bool.alpha", "true";
get "bool.beta", "false";
get "bool.gamma", "null";

put "string.alpha", "\"foo\"";
put "string.beta", "\"bar\"";
put "string.gamma", "\"baz\"";
get "string.alpha", "\"foo\"";
get "string.beta", "\"bar\"";
get "string.gamma", "\"baz\"";
put "string.alpha", "\"quux\"";
del "string.beta";
get "string.alpha", "\"quux\"";
get "string.beta", "null";
get "string.gamma", "\"baz\"";

put "uint.alpha", "12";
put "uint.beta", "17";
put "uint.gamma", "774";
get "uint.alpha", "12";
get "uint.beta", "17";
get "uint.gamma", "774";
put "uint.gamma", "929";
del "uint.alpha";
get "uint.alpha", "null";
get "uint.beta", "17";
get "uint.gamma", "929";

put "sint.alpha", "-12";
put "sint.beta", "+17";
put "sint.gamma", "-774";
get "sint.alpha", "-12";
get "sint.beta", "+17";
get "sint.gamma", "-774";
put "sint.gamma", "+924";
del "sint.alpha";
get "sint.alpha", "null";
get "sint.beta", "+17";
get "sint.gamma", "+924";

$dancer->kill();

1;
