# Copyright (C) 2002 Andrew Suffield <asuffield@users.sourceforge.net>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA

use strict;
use 5.006;
use warnings;
use DejaPerl qw/:framework/;
use Dancer::CmdPipe;
use Dancer::Spawn;

my $dancer = new Dancer::Spawn "database";
$dancer->install();
$dancer->start(modules => [qw/basic.logfile database.db test.database/]);
my $pipe = new Dancer::CmdPipe $dancer, "database", "test.database";

sub open_database($$)
  {
    my ($database, $name) = @_;
    $pipe->do("OPEN_DATABASE $database $name", "open database $database as $name", undef, [caller]);
  }

sub close_database($ )
  {
    my ($name) = @_;
    $pipe->do("CLOSE_DATABASE $name", "close database $name", undef, [caller]);
  }

sub associate_database($$)
  {
    my ($primary, $secondary) = @_;
    $pipe->do("ASSOCIATE_DATABASE $primary $secondary", "associate databases $primary and $secondary", undef, [caller]);
  }

sub put_value($$$ )
  {
    my ($name, $key, $value) = @_;
    $pipe->do("PUT_VALUE $name $key $value", "put key $key", undef, [caller]);
  }

sub get_value($$$ )
  {
    my ($name, $key, $value) = @_;
    $pipe->check_cmd_value("GET_VALUE $name $key", $value, "get key $key", undef, [caller]);
  }

sub delete_value($$)
  {
    my ($name, $key) = @_;
    $pipe->do("DELETE_VALUE $name $key", "delete key $key", undef, [caller]);
  }

verbose "Starting test run...";

$pipe->check_cmd_value("DRIVER_NAME", "db", "get driver name");
open_database "test/alpha", "db1";
put_value "db1", $_->[0], $_->[1] foreach ([alpha => 'a'], [beta => 'b'], [gamma => 'g']);
get_value "db1", $_->[0], $_->[1] foreach ([alpha => 'a'], [beta => 'b'], [gamma => 'g']);
close_database "db1";
open_database "test/alpha", "db1";
get_value "db1", $_->[0], $_->[1] foreach ([alpha => 'a'], [beta => 'b'], [gamma => 'g']);
close_database "db1";

open_database "test/beta", "db1";
put_value "db1", $_->[0], $_->[1] foreach ([a => 'alpha'], [b => 'beta'], [g => 'gamma']);
delete_value "db1", "a";
get_value "db1", a => '(none)';
close_database "db1";

open_database "test/gamma", "primary";
open_database "test/delta", "secondary";
associate_database "primary", "secondary";
foreach ([qw/alpha iti/], [qw/beta ni/], [qw/gamma san/])
  {
    put_value "primary", "$_->[0]/$_->[1]", "$_->[0]$_->[1]";
  }
foreach ([qw/alpha iti/], [qw/beta ni/], [qw/gamma san/])
  {
    get_value "primary", "$_->[0]/$_->[1]", "$_->[0]$_->[1]";
    get_value "secondary", "$_->[0]", "$_->[0]$_->[1]";
  }
close_database "primary";
close_database "secondary";

verbose "Finished test run";

$dancer->kill();

1;
