# Copyright (C) 2002 Andrew Suffield <asuffield@users.sourceforge.net>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA

use strict;
use 5.006;
use warnings;
use DejaPerl qw/:framework/;
use Dancer::CmdPipe;
use Dancer::Spawn;

my $dancer = new Dancer::Spawn "main.dbuffer";
$dancer->install();
$dancer->start(modules => [qw/basic.logfile test.main.dbuffer/]);
my $pipe = new Dancer::CmdPipe $dancer, "main.dbuffer", "test.main.dbuffer", sanity_checks => 1;

verbose "Starting test run...";

# First some stuff with short text
$pipe->do("NEW_DBUFFER", "create buffer");
$pipe->do("ADD foo", "add text");
$pipe->do("ADD bar", "add text");
$pipe->do("ADD baz", "add text");
$pipe->check_cmd_value("LENGTH", "9", "get length");
$pipe->check_cmd_value("GET 4", "foob", "get text");
$pipe->check_cmd_value("GET 6", "foobar", "get text");
$pipe->check_cmd_value("GET 0", "", "get no text");
$pipe->check_cmd_value("GET 1", "f", "get text");
$pipe->check_cmd_value("GET 9", "foobarbaz", "get text");
$pipe->check_cmd_value("GET 10", "foobarbaz", "get too much text");
$pipe->do("ADD quux", "add text");
$pipe->check_cmd_value("LENGTH", "13", "get length");
$pipe->check_cmd_value("GET 13", "foobarbazquux", "get text");
$pipe->check_cmd_value("GET 14", "foobarbazquux", "get too much text");
$pipe->check_cmd_value("FIND o", "1", "find character");
$pipe->check_cmd_value("FIND q", "9", "find character");
$pipe->check_cmd_value("FIND f", "0", "find character");
$pipe->check_cmd_value("FIND x", "12", "find character");
$pipe->check_cmd_value("FIND v", "-1", "find non-existant character");
$pipe->do("ADD ", "add no text");
$pipe->check_cmd_value("LENGTH", "13", "get length");
$pipe->check_cmd_value("GET 13", "foobarbazquux", "get text");
$pipe->check_cmd_value("GET 14", "foobarbazquux", "get too much text");
$pipe->do("REMOVE 4", "remove text");
$pipe->check_cmd_value("LENGTH", "9", "get length");
$pipe->check_cmd_value("GET 3", "arb", "get text");
$pipe->check_cmd_value("GET 20", "arbazquux", "get text");
$pipe->check_cmd_value("EXTRACT 4", "arba", "extract text");
$pipe->check_cmd_value("LENGTH", "5", "get length");
$pipe->check_cmd_value("GET 3", "zqu", "get text");
$pipe->check_cmd_value("GET 14", "zquux", "get text");
$pipe->check_cmd_value("EXTRACT 5", "zquux", "extract text");
$pipe->check_cmd_value("LENGTH", "0", "get length");
$pipe->check_cmd_value("GET 14", "", "get text");
$pipe->do("ADD 0123456789", "add text");
$pipe->check_cmd_value("LENGTH", "10", "get length");
$pipe->check_cmd_value("EXTRACT 6", "012345", "extract text");
$pipe->check_cmd_value("LENGTH", "4", "get length");
$pipe->check_cmd_value("EXTRACT 6", "6789", "extract text");
$pipe->check_cmd_value("LENGTH", "0", "get length");
$pipe->check_cmd_value("GET 1", "", "get text");
$pipe->do("DESTROY_DBUFFER", "destroy buffer");

# OK, next comes the long string stuff
# First some unevenly sized strings
# This is 61 characters (prime)
my $buflen = 0;
my $chars = "123456789ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz";
# 73 is also prime, and neither of them divides into 4000 easily; this
# should run over the edges neatly
my $longchars = $chars x 73;
my $longlen = length $longchars;
$pipe->do("NEW_DBUFFER", "create buffer");
$pipe->do("ADD $longchars", "add text");
$buflen = length $longchars;
$pipe->check_cmd_value("LENGTH", $longlen, "get length");
$pipe->check_cmd_value("GET 0", "", "get no text");

my $start = 0;
my $str = $longchars;
sub get_range($)
  {
    my $length = shift;
    $length = $buflen if $length > $buflen;
    while (($start + $length) > length $str)
      {
        $str .= $longchars;
      }
    $pipe->check_cmd_value("GET $length", substr($str, $start, $length), "get text");
  }

sub extract_range($)
  {
    my $length = shift;
    $length = $buflen if $length > $buflen;
    while (($start + $length) > length $str)
      {
        $str .= $longchars;
      }
    $pipe->check_cmd_value("EXTRACT $length", substr($str, $start, $length), "extract text");
    $start += $length;
    $buflen -= $length;
  }

sub get_length($)
  {
    my $length = shift;
    $pipe->check_cmd_value("LENGTH", $length, "get length");
  }

get_range($_) foreach qw/3999 4000 4001 4002/;
extract_range($_) foreach qw/2000 2000/;
$pipe->do("ADD $longchars", "add text");
$buflen += length $longchars;
get_length($buflen);
get_range($_) foreach qw/3999 4000 4001 4002/;
$pipe->do("ADD $longchars", "add text");
$buflen += length $longchars;
get_length($buflen);
get_range($_) foreach qw/3999 4000 4001 4002 7999 8000 8001 8002/;
extract_range($_) foreach qw/8001 0 1 2 3/;
get_length($buflen);
get_range($_) foreach qw/3999 4000 4001 4002/;

$pipe->do("DESTROY_DBUFFER", "destroy buffer");

# This time around we just want to cause some serious memory allocation
$pipe->do("NEW_DBUFFER", "create buffer");

$longchars = "x" x 4000;
$pipe->do("ADD $longchars", "add 'x' x 4000") foreach (1..105);
$pipe->do("ADD " . "x" x 480000, "add 'x' x 480000");
$pipe->do("REMOVE 400000", "remove 400000");
$pipe->check_cmd_value("LENGTH", (105 * 4000) + (8 * 60000) - 400000, "get length");
$pipe->check_cmd_value("GET 4000", $longchars, "get 'x' x 4000");
# And now push it beyond DBUFFER_ELEMENTS_FREE_THRESHOLD
$pipe->do("ADD " . $longchars x 105, "add 'x' x 60000 x 105");
$pipe->do("DESTROY_DBUFFER", "destroy buffer");

verbose "Finished test run";

$dancer->kill();

1;
