# Copyright (C) 2002 Andrew Suffield <asuffield@users.sourceforge.net>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA

use strict;
use 5.006;
use warnings;
use DejaPerl qw/:framework/;
use Dancer::CmdPipe;
use Dancer::Spawn;

my $dancer = new Dancer::Spawn "main.llist";
$dancer->install();
$dancer->start(modules => [qw/test.main.llist/]);
my $pipe = new Dancer::CmdPipe $dancer, "main.llist", "test.main.llist", sanity_checks => 1;

sub check_output_list(@)
  {
    $pipe->check_cmd_value("OUTPUT_LIST", join("\n", @_), "output " . join(' ', @_));
  }

sub check_output($)
  {
    my $value = shift;
    $pipe->check_cmd_value("OUTPUT", "$value", "output $value");
  }

verbose "Starting test run...";

$pipe->do("NEW_LLIST", "create list");
# OK, first, let's do some basic adding and removal
$pipe->do("ADD_TAIL alpha", "add alpha at tail");
$pipe->do("ADD_TAIL beta", "add beta at tail");
$pipe->do("ADD_TAIL gamma", "add gamma at tail");
check_output_list qw/alpha beta gamma/;
$pipe->do("ADD_HEAD delta", "add delta at head");
check_output_list qw/delta alpha beta gamma/;
$pipe->do("ADD_TAIL epsilon", "add epsilon at tail");
check_output_list qw/delta alpha beta gamma epsilon/;
# Iterate the list a bit and check it's right
$pipe->do("SEEK_HEAD", "seek to head");
check_output "delta";
$pipe->do("NEXT", "seek to next node");
check_output "alpha";
$pipe->do("NEXT", "seek to next node");
check_output "beta";
$pipe->do("NEXT", "seek to next node");
check_output "gamma";
$pipe->do("PREV", "seek to previous node");
check_output "beta";
$pipe->do("NEXT", "seek to next node");
check_output "gamma";
$pipe->do("NEXT", "seek to next node");
check_output "epsilon";
$pipe->do("PREV", "seek to previous node");
check_output "gamma";
$pipe->do("PREV", "seek to previous node");
check_output "beta";
$pipe->do("PREV", "seek to previous node");
check_output "alpha";
$pipe->do("PREV", "seek to previous node");
check_output "delta";
# Now some insertions
$pipe->do("SEEK_HEAD", "seek to head");
$pipe->do("NEXT", "seek to next node");
$pipe->do("NEXT", "seek to next node");
$pipe->do("INSERT_BEFORE eta", "insert element before current node");
check_output_list qw/delta alpha eta beta gamma epsilon/;
$pipe->do("INSERT_BEFORE theta", "insert element before current node");
check_output_list qw/delta alpha eta theta beta gamma epsilon/;
$pipe->do("NEXT", "seek to next node");
$pipe->do("INSERT_AFTER iota", "insert element after current node");
check_output_list qw/delta alpha eta theta beta gamma iota epsilon/;
# And insert a bit at the beginning and end
$pipe->do("SEEK_TAIL", "seek to tail");
$pipe->do("INSERT_AFTER kappa", "insert element after current node");
check_output_list qw/delta alpha eta theta beta gamma iota epsilon kappa/;
$pipe->do("INSERT_AFTER lamda", "insert element after current node");
check_output_list qw/delta alpha eta theta beta gamma iota epsilon lamda kappa/;
$pipe->do("SEEK_TAIL", "seek to tail");
$pipe->do("INSERT_AFTER mu", "insert element after current node");
check_output_list qw/delta alpha eta theta beta gamma iota epsilon lamda kappa mu/;
$pipe->do("SEEK_HEAD", "seek to head");
$pipe->do("INSERT_BEFORE nu", "insert element before current node");
check_output_list qw/nu delta alpha eta theta beta gamma iota epsilon lamda kappa mu/;
$pipe->do("INSERT_BEFORE xi", "insert element before current node");
check_output_list qw/nu xi delta alpha eta theta beta gamma iota epsilon lamda kappa mu/;
$pipe->do("SEEK_HEAD", "seek to head");
$pipe->do("INSERT_BEFORE omicron", "insert element before current node");
check_output_list qw/omicron nu xi delta alpha eta theta beta gamma iota epsilon lamda kappa mu/;
# Now some deletions
$pipe->do("SEEK_HEAD", "seek to head");
$pipe->do("DELETE", "delete current node");
check_output_list qw/nu xi delta alpha eta theta beta gamma iota epsilon lamda kappa mu/;
$pipe->do("SEEK_HEAD", "seek to head");
$pipe->do("DELETE", "delete current node");
check_output_list qw/xi delta alpha eta theta beta gamma iota epsilon lamda kappa mu/;
$pipe->do("SEEK_HEAD", "seek to head");
$pipe->do("NEXT", "seek to next node");
$pipe->do("NEXT", "seek to next node");
$pipe->do("DELETE", "delete current node");
check_output_list qw/xi delta eta theta beta gamma iota epsilon lamda kappa mu/;
$pipe->do("SEEK_TAIL", "seek to tail");
$pipe->do("DELETE", "delete current node");
check_output_list qw/xi delta eta theta beta gamma iota epsilon lamda kappa/;
$pipe->do("SEEK_TAIL", "seek to tail");
$pipe->do("DELETE", "delete current node");
check_output_list qw/xi delta eta theta beta gamma iota epsilon lamda/;
$pipe->do("SEEK_TAIL", "seek to tail");
$pipe->do("DELETE", "delete current node");
check_output_list qw/xi delta eta theta beta gamma iota epsilon/;
$pipe->do("SEEK_TAIL", "seek to tail");
$pipe->do("PREV", "seek to previous node");
$pipe->do("DELETE", "delete current node");
check_output_list qw/xi delta eta theta beta gamma epsilon/;
$pipe->do("SEEK_TAIL", "seek to tail");
$pipe->do("PREV", "seek to previous node");
$pipe->do("PREV", "seek to previous node");
$pipe->do("PREV", "seek to previous node");
$pipe->do("DELETE", "delete current node");
check_output_list qw/xi delta eta beta gamma epsilon/;
$pipe->do("DESTROY_LLIST", "destroy list");

verbose "Finished test run";

$dancer->kill();

1;
