# Copyright (C) 2002 Andrew Suffield <asuffield@users.sourceforge.net>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA

use strict;
use 5.006;
use warnings;
use DejaPerl qw/:framework/;
use Dancer::CmdPipe;
use Dancer::Spawn;

my $dancer = new Dancer::Spawn "main.strip_spaces";
$dancer->install();
$dancer->start(modules => [qw/test.main/]);
my $pipe = new Dancer::CmdPipe $dancer, "main.strip_spaces", "test.main", sanity_checks => 0;

my %flip = ('>' => '<', '<' => '>', '==' => '==');

verbose "Starting test run...";

while (<DATA>)
  {
    chomp;
    s/\#.*//;
    my ($ver_a, $sigil, $ver_b, undef) = split / /;
    $pipe->check_cmd_value("XSTRVERSCMP $ver_a $ver_b", $sigil, "compare $_");
    # Check that xstrverscmp() is inverse-reflexive
    $pipe->check_cmd_value("XSTRVERSCMP $ver_b $ver_a", $flip{$sigil}, "inverse compare $_");
  }

close DATA;

verbose "Finished test run";

$dancer->kill();

1;

__DATA__
aaa < bbb # lexicographic comparison
a < aa # absence of a character sorts lowest
1 < 10 # numeric comparison
a1 < a10 # composite
a10 < aa1 # lexicographic by components
 == # two empty strings
a == a
a1 == a01
