# Copyright (C) 2002 Andrew Suffield <asuffield@users.sourceforge.net>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA

use strict;
use 5.006;
use warnings;
use DejaPerl qw/:framework/;
use Dancer::CmdPipe;
use Dancer::Spawn;

my $dancer = new Dancer::Spawn "attributes";
$dancer->install();
$dancer->put_config("protocol.max-unknown-send-queue", "integer", 4);
$dancer->put_config("transport.classes", "string", "tclass1");
$dancer->put_config("transport.classes.tclass1.transport", "string", "test");
$dancer->start(modules => [qw/database.db config.database transport protocol test.transport test.protocol/]);

my $trans = new Dancer::CmdPipe $dancer, "transport", "test.transport", sanity_checks => 0;
my $proto = new Dancer::CmdPipe $dancer, "protocol", "test.protocol", sanity_checks => 0;

verbose "Starting test run...";

# Setup a couple of connections
$trans->check_cmd_value("REGISTER", "INIT_TCLASS tclass1", "register transport");
$proto->do("REGISTER_PROTOCOL", "register protocol");
$proto->do("IDENTIFY_RESULT 1", "set identify result to 1 (ours)");
$trans->do("CONNECT tclass1 conn1", "create connection 1");
$trans->do("CONNECT tclass1 conn2", "create connection 2");
$trans->do("READ conn1 alpha", "send alpha to conn1 transport");
$proto->check_value("IDENTIFY: alpha", "identify from protocol");
$proto->check_value("NEW_SESSION", "new session from protocol");
$proto->check_value("INIT_SESSION", "init session from protocol");
$proto->check_value("READ_DATA: alpha", "read alpha from protocol");
$proto->do("NAME_SESSION conn1", "name session conn1");
$trans->do("READ conn2 alpha", "send alpha to conn2 transport");
$proto->check_value("IDENTIFY: alpha", "identify from protocol");
$proto->check_value("NEW_SESSION", "new session from protocol");
$proto->check_value("INIT_SESSION", "init session from protocol");
$proto->check_value("READ_DATA: alpha", "read alpha from protocol");
$proto->do("NAME_SESSION conn2", "name session conn2");

# Register some attributes
my $i = 1;
my %attr;
foreach (qw/alpha beta gamma delta/)
  {
    $proto->check_cmd_value("REGISTER_ATTRIBUTE $_", $i, "register attribute $_");
    $attr{$_} = $i++;
  }
# And reregister them to check
$proto->check_cmd_value("REGISTER_ATTRIBUTE $_", $attr{$_}, "reregister attribute $_") foreach keys %attr;

# They should all be NULL on both sessions initially
my %values = (0 => {map {$_ => "(null)"} keys %attr},
              1 => {map {$_ => "(null)"} keys %attr},
             );
sub check_values()
  {
    foreach my $conn (qw/conn1 conn2/)
      {
        foreach my $attr (keys %{$values{$conn}})
          {
            $proto->check_cmd_value("GET_ATTRIBUTE $conn $attr{$attr}", $values{$conn}{$attr},
                                    "get attribute $attr on session $conn");
          }
      }
  }

sub set_value(@)
  {
    my ($conn, $attr, $value) = @_;
    $proto->do("SET_ATTRIBUTE $conn $attr{$attr} $value", "set attribute $attr on session $conn");
    $values{$conn}{$attr} = $value;
  }

check_values;

# Now set and check for each of these
foreach ([conn2 => alpha => "iti"],
         [conn2 => alpha => "ni"],
         [conn1 => beta => "san"],
         [conn1 => gamma => "si"],
         [conn1 => delta => "go"],
         [conn2 => beta => "roku"],
         [conn2 => beta => "siti"],
        )
  {
    set_value @$_;
    check_values;
  }

verbose "Finished test run";

$trans->kill;
$proto->kill;

$dancer->kill();

1;
