# Copyright (C) 2002 Andrew Suffield <asuffield@users.sourceforge.net>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA

use strict;
use 5.006;
use warnings;
use DejaPerl qw/:framework/;
use Dancer::CmdPipe;
use Dancer::Spawn;

my $dancer = new Dancer::Spawn "message_type";
$dancer->install();
$dancer->put_config("protocol.max-unknown-send-queue", "integer", 4);
$dancer->put_config("transport.classes", "string", "");
$dancer->start(modules => [qw/database.db config.database transport protocol test.message/]);

my $pipe = new Dancer::CmdPipe $dancer, "message_type", "test.message", sanity_checks => 0;

verbose "Starting test run...";

$pipe->check_cmd_value("GET_TYPE invalid", "-1", "get invalid message type");
$pipe->check_cmd_value("GET_TYPE_NAME 0", "(null)", "get invalid message type name");
$pipe->check_cmd_value("GET_TYPE_NAME -1", "(null)", "get invalid message type name");
$pipe->check_cmd_value("REGISTER_TYPE alpha", "0", "register message type alpha");
$pipe->check_cmd_value("REGISTER_TYPE beta", "1", "register message type beta");
$pipe->check_cmd_value("REGISTER_TYPE gamma", "2", "register message type gamma");
$pipe->check_cmd_value("REGISTER_TYPE delta", "3", "register message type delta");
$pipe->check_cmd_value("REGISTER_TYPE alpha", "0", "reregister message type alpha");
$pipe->check_cmd_value("GET_TYPE alpha", "0", "get message type alpha");
$pipe->check_cmd_value("GET_TYPE beta", "1", "get message type beta");
$pipe->check_cmd_value("GET_TYPE gamma", "2", "get message type gamma");
$pipe->check_cmd_value("GET_TYPE delta", "3", "get message type delta");
$pipe->check_cmd_value("GET_TYPE_NAME 0", "alpha", "get message type name for 0");
$pipe->check_cmd_value("GET_TYPE_NAME 1", "beta", "get message type name for 1");
$pipe->check_cmd_value("GET_TYPE_NAME 2", "gamma", "get message type name for 2");
$pipe->check_cmd_value("GET_TYPE_NAME 3", "delta", "get message type name for 3");
$pipe->check_cmd_value("GET_TYPE invalid", "-1", "get invalid message type");
$pipe->check_cmd_value("GET_TYPE_NAME 5", "(null)", "get invalid message type name");
$pipe->check_cmd_value("GET_TYPE_NAME -1", "(null)", "get invalid message type name");

# Now register enough types to trigger allocation of some more memory
my $x = 4;
foreach (1..64)
  {
    $pipe->check_cmd_value("REGISTER_TYPE type$_", $x++, "register message type type$_");
  }

# And check a few of these again
$pipe->check_cmd_value("REGISTER_TYPE alpha", "0", "reregister message type alpha");
$pipe->check_cmd_value("GET_TYPE alpha", "0", "get message type alpha");
$pipe->check_cmd_value("GET_TYPE beta", "1", "get message type beta");
$pipe->check_cmd_value("GET_TYPE gamma", "2", "get message type gamma");
$pipe->check_cmd_value("GET_TYPE delta", "3", "get message type delta");
$pipe->check_cmd_value("GET_TYPE_NAME 0", "alpha", "get message type name for 0");
$pipe->check_cmd_value("GET_TYPE_NAME 1", "beta", "get message type name for 1");
$pipe->check_cmd_value("GET_TYPE_NAME 2", "gamma", "get message type name for 2");
$pipe->check_cmd_value("GET_TYPE_NAME 3", "delta", "get message type name for 3");
$pipe->check_cmd_value("GET_TYPE invalid", "-1", "get invalid message type");

$x = 4;
foreach (1..64)
  {
    $pipe->check_cmd_value("GET_TYPE type$_", $x++, "get message type type$_");
  }

verbose "Finished test run";

$dancer->kill();

1;
