/************************************************************************
 *   This is part of Dancer, a text relay daemon
 *   Copyright (C) 2002 Andrew Suffield <asuffield@users.sourceforge.net>
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 */

include router_setup;

/********************************************
 * Errors caught in the first level handler *
 ********************************************/

/************************************
 * Before handshaking has completed *
 ************************************/

query_node_information_type = call get_message_type "query node information";
accept_handshake_type = call get_message_type "accept handshake";

call create_connection "conn1";

/* 'query node information' can only be used after handshaking has completed */
write proto message: read: conn1  type: query node information  str: alpha;
read proto message:               type: handshake required      origin: router.
                                uint32: $query_node_information_type;
call no_messages;

/* messages that require RMC_ORIGIN will always bounce without it */
write proto message: read: conn1  type: accept handshake;
read proto message:               type: wrong parameters        origin: router.
                                   str: No origin specified
                                uint32: $accept_handshake_type;
call no_messages;

/* as will those that require RMC_DEST and don't have it */
write proto message: read: conn1  type: accept handshake        origin: alpha;
read proto message:               type: wrong parameters        origin: router.
                                   str: No destination specified
                                uint32: $accept_handshake_type;
call no_messages;

call close_connection "conn1";

/*********************
 * After handshaking *
 *********************/

call create_connection "conn1";
call handshake "conn1" "alpha" "Real name alpha";

call create_connection "conn2";
call handshake "conn2" "r_beta" "Real name beta (router)";
write proto message: read: conn2  type: query routable nodes  origin: r_beta  dest: router.;
write proto message: read: conn2  type: done routable nodes   origin: r_beta  dest: router.;
read proto message:               type: routable node         origin: router. dest: r_beta
                                   str: alpha\\,Real name alpha;
read proto message:               type: done routable nodes   origin: router. dest: r_beta;

/* all messages without origin after handshaking will bounce */
/* This is no longer true, unless the peer is a router */
write proto message: read: conn1  type: query node information  str: alpha;
read proto message:               type: node information        origin: router.  dest: alpha
                                   str: alpha\\,(null)\\,Real name alpha
                                uint32: 1;
call no_messages;

write proto message: read: conn2  type: query node information  str: alpha;
read proto message:               type: wrong parameters        origin: router.  dest: r_beta
                                   str: No origin specified
                                uint32: $query_node_information_type;
call no_messages;

/* messages from unknown origins will bounce */
write proto message: read: conn2  type: query node information  origin: beta     dest: beta     str: beta;
read proto message:               type: wrong parameters        origin: router.  dest: r_beta
                                   str: Origin not known
                                uint32: $query_node_information_type;
call no_messages;

/* Get the other router out of the way */
call close_connection "conn2";

/* messages for unknown destinations will bounce */
write proto message: read: conn1  type: query node information  origin: alpha    dest: beta     str: beta;
read proto message:               type: invalid destination     origin: router.  dest: alpha
                                   str: beta
                                uint32: $query_node_information_type;
call no_messages;

/* messages travelling in the wrong direction are fatal errors */
write proto message: read: conn1  type: query node information  origin: alpha    dest: alpha    str: beta;
read proto message:               type: fatal error             origin: router.  dest: alpha
                                   str: Received message travelling in the wrong direction;
call no_messages;
call router_closed_connection "conn1";

/* These currently cause the immediate close of the session */
call create_connection "conn1";
call handshake "conn1" "alpha" "Real name alpha";
write proto message: read: conn1  type: reject set name         origin: alpha    dest: router.  str: beta;
read proto message:               type: fatal error             origin: router.  dest: alpha;
call no_messages;
call router_closed_connection "conn1";

call create_connection "conn1";
call handshake "conn1" "alpha" "Real name alpha";
write proto message: read: conn1  type: reject set real name    origin: alpha    dest: router.  str: beta;
read proto message:               type: fatal error             origin: router.  dest: alpha;
call no_messages;
call router_closed_connection "conn1";

kill "router";
