/************************************************************************
 *   This is part of Dancer, a text relay daemon
 *   Copyright (C) 2002 Andrew Suffield <asuffield@users.sourceforge.net>
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 */

include router_setup;

query_node_information_type = call get_message_type "query node information";

/************************
 * Successful handshake *
 ************************/

/* Simple handshake */
call create_connection "conn1";
write proto message: read: conn1  type: set name       str: alpha;
call no_messages;
write proto message: read: conn1  type: set real name  str: real name 1;
call no_messages;
write proto message: read: conn1  type: done handshake;
read proto message:               type: set name       dest: alpha str: router.;
read proto message:               type: set real name  dest: alpha str: Router;
read proto message:               type: done handshake    origin: router.  dest: alpha;
read proto message:               type: accept handshake  origin: router.  dest: alpha;
call no_messages;
write proto message: read: conn1  type: accept handshake  origin: alpha    dest: router.;
read proto message:               type: unregistered user origin: router.  dest: alpha;
call no_messages;
call close_connection "conn1";

/* Simple handshake with the sequence reversed */
call create_connection "conn1";
write proto message: read: conn1  type: set real name  str: real name 1;
call no_messages;
write proto message: read: conn1  type: set name       str: alpha;
call no_messages;
write proto message: read: conn1  type: accept handshake  origin: alpha    dest: router.;
call no_messages;
write proto message: read: conn1  type: done handshake;
read proto message:               type: set name       dest: alpha str: router.;
read proto message:               type: set real name  dest: alpha str: Router;
read proto message:               type: done handshake    origin: router.  dest: alpha;
read proto message:               type: accept handshake  origin: router.  dest: alpha;
read proto message:               type: unregistered user origin: router.  dest: alpha;
call no_messages;
call close_connection "conn1";

/* Simple handshake, don't send an accept message before closing */
call create_connection "conn1";
write proto message: read: conn1  type: set name       str: alpha;
call no_messages;
write proto message: read: conn1  type: set real name  str: real name 1;
call no_messages;
write proto message: read: conn1  type: done handshake;
read proto message:               type: set name       dest: alpha str: router.;
read proto message:               type: set real name  dest: alpha str: Router;
read proto message:               type: done handshake    origin: router.  dest: alpha;
read proto message:               type: accept handshake  origin: router.  dest: alpha;
call no_messages;
call close_connection "conn1";

/* Simple handshake, don't send an done message before closing */
call create_connection "conn1";
write proto message: read: conn1  type: set name       str: alpha;
call no_messages;
write proto message: read: conn1  type: set real name  str: real name 1;
call no_messages;
write proto message: read: conn1  type: accept handshake  origin: alpha    dest: router.;
call no_messages;
call close_connection "conn1";

/* Simple handshake, don't send an accept or done message before closing */
call create_connection "conn1";
write proto message: read: conn1  type: set name       str: alpha;
call no_messages;
write proto message: read: conn1  type: set real name  str: real name 1;
call no_messages;
call close_connection "conn1";

/* Simple handshake, other sequence, don't send an accept or done message before closing */
call create_connection "conn1";
write proto message: read: conn1  type: set real name  str: real name 1;
call no_messages;
write proto message: read: conn1  type: set name       str: alpha;
call no_messages;
call close_connection "conn1";

/* No real name specified */
call create_connection "conn1";
write proto message: read: conn1  type: set name       str: alpha;
call no_messages;
write proto message: read: conn1  type: accept handshake  origin: alpha    dest: router.;
call no_messages;
call close_connection "conn1";

/* Nothing at all */
call create_connection "conn1";
call no_messages;
call close_connection "conn1";

/**********************
 * Handshaking errors *
 **********************/

/* Protocol errors: 'done handshake' out of sequence is ignored */
call create_connection "conn1";
write proto message: read: conn1  type: set real name  str: real name 1;
call no_messages;
write proto message: read: conn1  type: done handshake;
call no_messages;
call close_connection "conn1";

call create_connection "conn1";
write proto message: read: conn1  type: set name       str: alpha;
call no_messages;
write proto message: read: conn1  type: done handshake;
call no_messages;
call close_connection "conn1";

/* Protocol errors: abort on missing name */
call create_connection "conn1";
write proto message: read: conn1  type: set real name  str: real name 1;
call no_messages;
write proto message: read: conn1  type: accept handshake  origin: alpha    dest: router.;
read proto message:               type: fatal error       origin: router.
                                   str: Handshake failed (your name was not set);
call no_messages;
call router_closed_connection "conn1";

/* Protocol errors: abort on mismatched name */
call create_connection "conn1";
write proto message: read: conn1  type: set name       str: alpha;
call no_messages;
write proto message: read: conn1  type: accept handshake  origin: beta     dest: router.;
read proto message:               type: fatal error       origin: router.  dest: alpha
                                   str: Handshake failed (your name does not match);
call no_messages;
call router_closed_connection "conn1";

call create_connection "conn1";
write proto message: read: conn1  type: set name       str: alpha;
call no_messages;
write proto message: read: conn1  type: accept handshake  origin: alpha    dest: gamma;
read proto message:               type: fatal error       origin: router.  dest: alpha
                                   str: Handshake failed (my name does not match);
call no_messages;
call router_closed_connection "conn1";

/* This one is more complex. If we change name after
 * 'accept handshake', then our 'accept' is dropped, and the handshake
 * is not complete
 */

call create_connection "conn1";
write proto message: read: conn1  type: set real name           str: real name 1;
call no_messages;
write proto message: read: conn1  type: set name                str: alpha;
call no_messages;
write proto message: read: conn1  type: accept handshake     origin: alpha   dest: router.;
call no_messages;
write proto message: read: conn1  type: set name                str: beta;
read proto message:               type: set name             origin: alpha   dest: alpha
                                                                str: beta;
call no_messages;

/* Now, the server will send *its* accept, but will wait for ours
 * before completing
 */
write proto message: read: conn1  type: done handshake;
read proto message:               type: set name       dest: beta  str: router.;
read proto message:               type: set real name  dest: beta  str: Router;
read proto message:               type: done handshake       origin: router.  dest: beta;
read proto message:               type: accept handshake     origin: router.  dest: beta;
call no_messages;
write proto message: read: conn1  type: query node information  str: alpha;
read proto message:               type: handshake required   origin: router.  dest: beta
                                uint32: $query_node_information_type;
call no_messages;

/* After we actually accept with our new name, the server completes */
write proto message: read: conn1  type: accept handshake     origin: beta     dest: router.;
read proto message:               type: unregistered user    origin: router.  dest: beta;
call no_messages;
write proto message: read: conn1  type: query node information  origin: beta  dest: router.
                                                                str: alpha;
read proto message:               type: node information     origin: router.  dest: beta
                                uint32: 0                       str: alpha;
call no_messages;


kill "router";
