/************************************************************************
 *   This is part of Dancer, a text relay daemon
 *   Copyright (C) 2002 Andrew Suffield <asuffield@users.sourceforge.net>
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 */

include router_setup;

/****************************
 * Assorted simple messages *
 ****************************/

/*****************
 * close session *
 *****************/
call create_connection "conn1" 0;
call handshake "conn1" "alpha" "Real name alpha";
write proto message: read: conn1  type: close session           origin: alpha   dest: router.  str: router.;
call no_messages;
call router_closed_connection "conn1";

/**************************
 * query node information *
 **************************/
/* target not present */
call create_connection "conn1" 0;
call handshake "conn1" "alpha" "Real name alpha";
write proto message: read: conn1  type: query node information  origin: alpha   dest: router.
                                                                   str: beta;
read proto message:               type: node information        origin: router. dest: alpha
                                                                   str: beta
                                                                uint32: 0;
call no_messages;

/* target present */
call create_connection "conn2" 1;
call handshake "conn2" "beta" "Real name beta";
write proto message: read: conn1  type: query node information  origin: alpha   dest: router.
                                                                   str: beta;
read proto message:               type: node information        origin: router. dest: alpha
                                                                   str: beta\\,(null)\\,Real name beta
                                                                uint32: 1;
call no_messages;

/*  target closed connection, no longer present */
call close_connection "conn2";
write proto message: read: conn1  type: query node information  origin: alpha   dest: router.
                                                                   str: beta;
read proto message:               type: node information        origin: router. dest: alpha
                                                                   str: beta
                                                                uint32: 0;
call no_messages;

/* same behaviour if target sent 'close session' */
call create_connection "conn2" 1;
call handshake "conn2" "beta" "Real name beta";
write proto message: read: conn2  type: close session           origin: beta    dest: router.  str: router.;
call no_messages;
call router_closed_connection "conn2";
write proto message: read: conn1  type: query node information  origin: alpha   dest: router.
                                                                   str: beta;
read proto message:               type: node information        origin: router. dest: alpha
                                                                   str: beta
                                                                uint32: 0;
call no_messages;
call close_connection "conn1";

/************
 * set name *
 ************/

call create_connection "conn1" 0;
call handshake "conn1" "alpha" "Real name alpha";
call create_connection "conn2" 1;
call handshake "conn2" "beta" "Real name beta";

/* name conflict, fail */
write proto message: read: conn1  type: set name         origin: alpha   dest: router.  str: beta;
read proto message:               type: reject set name  origin: router. dest: alpha    str: beta;
call no_messages;

/* no conflict, suceed */
write proto message: read: conn1  type: set name         origin: alpha   dest: router.  str: gamma;
read proto message:               type: set name         origin: alpha   dest: alpha    str: gamma;
call no_messages;

/* check that everything that should have been updated, was */
write proto message: read: conn2  type: query node information  origin: beta  dest: router.
                                                                   str: alpha;
read proto message:               type: node information        origin: router. dest: beta
                                                                   str: alpha
                                                                uint32: 0;
call no_messages;
write proto message: read: conn2  type: query node information  origin: beta  dest: router.
                                                                   str: beta;
read proto message:               type: node information        origin: router. dest: beta
                                                                   str: beta\\,(null)\\,Real name beta
                                                                uint32: 1;
call no_messages;
write proto message: read: conn2  type: query node information  origin: beta  dest: router.
                                                                   str: gamma;
read proto message:               type: node information        origin: router. dest: beta
                                                                   str: gamma\\,(null)\\,Real name alpha
                                                                uint32: 1;
call no_messages;

call close_connection "conn1";
call close_connection "conn2";

/********************
 * Message relaying *
 ********************/

call create_connection "conn1" 0;
call handshake "conn1" "alpha" "Real name alpha";
call create_connection "conn2" 1;
call handshake "conn2" "beta" "Real name beta";

/* relaying denied */
write proto message: read: conn1  type: query node information  origin: alpha   dest: beta
                                                                   str: gamma;
read proto message:               type: relaying denied         origin: router. dest: alpha
                                                                   str: beta;
call no_messages;

/* relaying accepted, 'private' message */
write proto message: read: conn1  type: private                 origin: alpha   dest: beta
                                                                   str: gamma;
read proto message:               type: private                 origin: alpha   dest: beta
                                                                   str: gamma;

call close_connection "conn1";
call close_connection "conn2";

/********************************
 * Miscellaneous server queries *
 ********************************/

call create_connection "conn1" 0;
call handshake "conn1" "alpha" "Real name alpha";
call create_connection "conn2" 1;
call handshake "conn2" "beta" "Real name beta";

/* This bit of perl acquires the file LICENSE, stripping everything after and including the -- */
license = eval '
my @license;
foreach (`cat $srcdir/../LICENSE`)
  {
    chomp;
    last if $_ eq "--";
    push @license, $_;
  }
return join("\\,", @license);
';
write proto message: read: conn1  type: query node license information  origin: alpha    dest: router.;
read proto message:               type: node license information        origin: router.  dest: alpha
                                   str: $license;

/* Shove a '\ ' before the :, to stop it parsing as a field. Hack. */
authors = eval 'join("\\,", (map {chomp; $_ =~ s/:/\\ :/g; $_} `cat $srcdir/../AUTHORS`))';
write proto message: read: conn1  type: query node author information   origin: alpha    dest: router.;
read proto message:               type: node author information         origin: router.  dest: alpha
                                   str: $authors;


call close_connection "conn1";
call close_connection "conn2";

kill "router";
