/************************************************************************
 *   This is part of Dancer, a text relay daemon
 *   Copyright (C) 2002 Andrew Suffield <asuffield@users.sourceforge.net>
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 */

preinstall "rfc1459.system.connect.1";
put_config "rfc1459.system.connect.1" "router.name" "string" "1.router.";
put_config "rfc1459.system.connect.1" "router.real_name" "string" "Router 1";
put_config "rfc1459.system.connect.1" "protocol.max-unknown-send-queue" "integer" "4";
put_config "rfc1459.system.connect.1" "transport.classes" "string" "tclass1";
put_config "rfc1459.system.connect.1" "transport.classes.tclass1.transport" "string" "ipv4.tcp";
put_config "rfc1459.system.connect.1" "transport.classes.tclass1.bind_address" "integer" (eval 'unpack "I", v127.0.0.1');
put_config "rfc1459.system.connect.1" "transport.classes.tclass1.bind_port" "integer" (eval 'unpack "S", pack "n", 6543');

preinstall "rfc1459.system.connect.2";
put_config "rfc1459.system.connect.2" "router.name" "string" "2.router.";
put_config "rfc1459.system.connect.2" "router.real_name" "string" "Router 2";
put_config "rfc1459.system.connect.2" "protocol.max-unknown-send-queue" "integer" "4";
put_config "rfc1459.system.connect.2" "transport.classes" "string" "tclass1";
put_config "rfc1459.system.connect.2" "transport.classes.tclass1.transport" "string" "ipv4.tcp";
put_config "rfc1459.system.connect.2" "transport.classes.tclass1.bind_address" "integer" (eval 'unpack "I", v127.0.0.1');
put_config "rfc1459.system.connect.2" "transport.classes.tclass1.bind_port" "integer" (eval 'unpack "S", pack "n", 6544');

spawn "rfc1459.system.connect.1" modules => "database.db" "config.database" "transport" "protocol"
                                            "transport.ipv4.tcp"
                                            "protocol.rfc1459.client" "protocol.dgmtp"
                                            "auth" "router";

spawn "rfc1459.system.connect.2" modules => "database.db" "config.database" "transport" "protocol"
                                            "transport.ipv4.tcp"
                                            "protocol.rfc1459.client" "protocol.dgmtp"
                                            "auth" "router";

open log1 log "rfc1459.system.connect.1";
open log2 log "rfc1459.system.connect.2";

open alpha1 rfc1459 "127.0.0.1" 6543;
open beta2 rfc1459 "127.0.0.1" 6544;

write alpha1 USER x x x :Real name alpha;
write alpha1 NICK alpha;
read alpha1 :1.router. 001 alpha :Welcome to the Internet Relay Network alpha;
read alpha1 :1.router. 002 alpha :Your host is *[*/0], running version 1.1.0-unreleased/protocol.rfc1459.client-1.0;
read alpha1 :1.router. 399 alpha :This username has not been registered. Please register.;

write beta2 USER x x x :Real name beta;
write beta2 NICK beta;
read beta2 :2.router. 001 beta :Welcome to the Internet Relay Network beta;
read beta2 :2.router. 002 beta :Your host is *[*/0], running version 1.1.0-unreleased/protocol.rfc1459.client-1.0;
read beta2 :2.router. 399 beta :This username has not been registered. Please register.;

/* At this stage, neither can see the other */
write alpha1 PRIVMSG beta :This is a message;
read alpha1 :1.router. 401 alpha beta :No such nick/channel;
write beta2 PRIVMSG alpha :This is a message;
read beta2 :2.router. 401 beta alpha :No such nick/channel;

/* Now connect the two servers together */
write alpha1 CONNECT dgmtp ipv4.tcp tclass1 127.0.0.1:6544;
read log1 router [info]: Connection to router 2.router. established;
read log2 router [info]: Connection to router 1.router. established;

write alpha1 PRIVMSG beta :This is a message;
read beta2 :alpha PRIVMSG beta :This is a message;

write beta2 PRIVMSG alpha :This is a message;
read alpha1 :beta PRIVMSG alpha :This is a message;

write beta2 QUIT;
eof beta2;

open gamma2 rfc1459 "127.0.0.1" 6544;
write gamma2 USER x x x :Real name gamma;
write gamma2 NICK gamma;
read gamma2 :2.router. 001 gamma :Welcome to the Internet Relay Network gamma;
read gamma2 :2.router. 002 gamma :Your host is *[*/0], running version 1.1.0-unreleased/protocol.rfc1459.client-1.0;
read gamma2 :2.router. 399 gamma :This username has not been registered. Please register.;

/* We need to give those messages time to propagate */
write gamma2 PING 1.router.;
read gamma2 :1.router. PONG gamma;

write alpha1 PRIVMSG beta :This is a message;
read alpha1 :1.router. 401 alpha beta :No such nick/channel;

write alpha1 PRIVMSG gamma :This is a message;
read gamma2 :alpha PRIVMSG gamma :This is a message;

write gamma2 PRIVMSG alpha :This is a message;
read alpha1 :gamma PRIVMSG alpha :This is a message;

open alpha2 rfc1459 "127.0.0.1" 6544;
write alpha2 USER x x x :Real name alpha;
write alpha2 NICK alpha;
read alpha2 :2.router. 433 * alpha :Nickname is already in use;
write alpha2 QUIT;
eof alpha2;

write alpha1 QUIT;
eof alpha1;

kill "rfc1459.system.connect.1";
kill "rfc1459.system.connect.2";
