/************************************************************************
 *   This is part of Dancer, a text relay daemon
 *   Copyright (C) 2002 Andrew Suffield <asuffield@users.sourceforge.net>
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 */

preinstall "transport";
put_config "transport" "transport.classes" "string" "tclass1,tclass2";
put_config "transport" "transport.classes.tclass1.transport" "string" "test";
put_config "transport" "transport.classes.tclass2.transport" "string" "test";
spawn "transport" modules => "database.db" "config.database" "transport" "protocol"
                             "test.protocol" "test.transport";

open trans cmdpipe "transport" "test.transport";
open proto cmdpipe "transport" "test.protocol";

/* test the tclass init and deinit stuff */
write trans REGISTER;
read trans INIT_TCLASS tclass1
INIT_TCLASS tclass2;
write trans UNREGISTER;
read trans DEACTIVATE_TCLASS tclass1 tclass1
DEACTIVATE_TCLASS tclass2 tclass2;
write trans REGISTER;
read trans INIT_TCLASS tclass1
INIT_TCLASS tclass2;
write trans UNREGISTER;
read trans DEACTIVATE_TCLASS tclass1 tclass1
DEACTIVATE_TCLASS tclass2 tclass2;

/* Same for protocol */
write proto REGISTER_PROTOCOL;
write proto UNREGISTER_PROTOCOL;
write proto REGISTER_PROTOCOL;
write proto UNREGISTER_PROTOCOL;

/* OK, let's register for real this time */
write trans REGISTER;
read trans INIT_TCLASS tclass1
INIT_TCLASS tclass2;

/* Make a few connections */
write trans CONNECT tclass1 conn1;
write trans CONNECT tclass1 conn2;
write trans CONNECT tclass2 conn3;
write trans CONNECT tclass2 conn4;

/* Check out the protocol identification stuff */

/* This one will be closed, because no protocols are available */
write trans READ conn1 alpha;
read trans ABORT conn1;
/* Reopen it */
write trans CONNECT tclass1 conn1;

/* Register the protocol */
write proto REGISTER_PROTOCOL;

write trans READ conn1 alpha;
read proto IDENTIFY: alpha;
write trans READ conn1 beta;
read proto IDENTIFY: alphabeta;

/* Now, set the identify result to be a "fail" and run it again */
write proto IDENTIFY_RESULT -1;
write trans READ conn1 gamma;
read trans ABORT conn1;
read proto IDENTIFY: alphabetagamma;
/* Reopen it */
write trans CONNECT tclass1 conn1;

/* Fail with nothing in the buffer */
write proto IDENTIFY_RESULT -1;
write trans READ conn1 alpha;
read trans ABORT conn1;
read proto IDENTIFY: alpha;
/* Reopen it */
write trans CONNECT tclass1 conn1;

/* Set the result to be "unknown" and fill the buffer */
write proto IDENTIFY_RESULT 0;
write trans READ conn1 alpha;
read proto IDENTIFY: alpha;
write trans READ conn1 beta;
read proto IDENTIFY: alphabeta;
/* Set the result to be "ours" and send some more data in */
write proto IDENTIFY_RESULT 1;
write trans READ conn1 gamma;
read proto IDENTIFY: alphabetagamma;
read proto NEW_SESSION;
write proto NAME_SESSION conn1;
read proto INIT_SESSION;
read proto READ_DATA: alphabetagamma;

/* And send a bit more data through */
write trans READ conn1 delta;
read proto READ_DATA: delta;
write trans READ conn1 epsilon;
read proto READ_DATA: epsilon;

/* Result is "ours", send some data into another connection */
write trans READ conn2 alpha;
read proto IDENTIFY: alpha;
read proto NEW_SESSION;
write proto NAME_SESSION conn2;
read proto INIT_SESSION;
read proto READ_DATA: alpha;
write trans READ conn2 beta;
read proto READ_DATA: beta;

/* Now send some data back the other way */
write proto WRITE_DATA conn1 iti;
read trans WRITE conn1: iti;
write proto WRITE_DATA conn1 ni;
read trans WRITE conn1: ni;
write proto WRITE_DATA conn1 san;
read trans WRITE conn1: san;
write proto WRITE_DATA conn2 shi;
read trans WRITE conn2: shi;
write proto WRITE_DATA conn2 go;
read trans WRITE conn2: go;

/* Now send a close each way */
write proto SHUTDOWN_SESSION conn1;
read trans CLOSE conn1;
write trans CLOSE conn1;
read proto CLOSE_SESSION;

/* Begin a new connection */
write proto BEGIN_CONNECTION test test tclass1 target;
read trans BEGIN target;
write trans FINISH target;
read proto CONNECTION_READY;
read proto NEW_OUTGOING_SESSION;
read proto READ_DATA: ;
write proto SHUTDOWN_SESSION target;
read trans CLOSE target;
write trans CLOSE target;
read proto CLOSE_SESSION;

kill "transport";
