# Copyright (C) 2002 Andrew Suffield <asuffield@users.sourceforge.net>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA

package Dancer::CmdScript;

use strict;
use 5.006;
use warnings;
use DejaPerl qw/:framework/;
use Dancer::Spawn;

use Carp;

BEGIN
  {
    use Exporter;
    our @ISA = qw/Exporter/;
    our $VERSION = "0.0.1";
    our @EXPORT = qw//;
    our @EXPORT_OK = qw//;
  }

sub new($%)
  {
    my $class = shift;
    my %args = @_;

    my $prefix = defined $args{category} ? ($args{category} . "-") : "";
    my $self = {
                category => $args{category} || undef,
                name => $args{name},
                block => $args{block},
                prefix => $prefix,
                lines => [],
                variables => $args{parent} ? $args{parent}->variables() : {},
                connections => $args{parent} ? $args{parent}->connections() : {},
                spawns => $args{parent} ? $args{parent}->spawns() : {},
                data => $args{data} ? $args{data}->data() : {},
                stack => [],
               };
    bless $self, $class;
    return $self;
  }

sub category
  {
    my $self = shift;
    return $self->{category};
  }

sub name
  {
    my $self = shift;
    return $self->{name};
  }

sub run
  {
    my $self = shift;
    my $ret;
    eval
      {
        $ret = $self->{block}->eval($self);
      };
    if ($@)
      {
        if (ref $@ eq 'HASH')
          {
            error $@->{msg}, 0, caller => [$self->{category}, $self->{name}, $@->{line}];
          }
        else
          {
            error $@, 0;
          }
        return undef;
      }
    else
      {
        return $ret;
      }
  }

sub prefix
  {
    my $self = shift;
    return $self->{prefix};
  }

sub variables
  {
    my $self = shift;
    return $self->{variables};
  }

sub connections
  {
    my $self = shift;
    return $self->{connections};
  }

sub spawns
  {
    my $self = shift;
    return $self->{spawns};
  }

sub data
  {
    my $self = shift;
    return $self->{data};
  }

sub stack
  {
    my $self = shift;
    return $self->{stack};
  }

sub interpolate_regexp
  {
    my $self = shift;
    my $str = shift;
    my $outstr = "";
    while (length $str)
      {
        if ($str =~ s/^((?:[^\$\\]|\\[^\$])+)//)
          {
            $outstr .= $1;
          }
        elsif ($str =~ s/^\\\$//)
          {
            $outstr .= "\$";
          }
        elsif ($str =~ s/^\$([_A-Za-z][_A-Za-z0-9]+)// or $str =~ s/^\$(\d+)//)
          {
            die "Undefined variable $1" unless exists $self->{variables}{$1};
            $outstr .= $self->{variables}{$1};
          }
        elsif ($str =~ s/^\$//)
          {
            $outstr .= "\$";
          }
      }
    return $outstr;
  }

sub interpolate_string
  {
    my $self = shift;
    my $str = shift;
    my $outstr = "";
    while (length $str)
      {
        if ($str =~ s/^([^\$\\]+)//)
          {
            $outstr .= $1;
          }
        elsif ($str =~ s/^\\\\//)
          {
            $outstr .= "\\";
          }
        elsif ($str =~ s/^\\\$//)
          {
            $outstr .= "\$";
          }
        elsif ($str =~ s/^\$([_A-Za-z][_A-Za-z0-9]+)// or $str =~ s/^\$(\d+)//)
          {
            die "Undefined variable $1" unless exists $self->{variables}{$1};
            $outstr .= $self->{variables}{$1};
          }
        elsif ($str =~ s/^\$//)
          {
            $outstr .= "\$";
          }
        else
          {
            die "Unknown escape sequence";
          }
      }
    return $outstr;
  }

sub cmdpipe
  {
    my $self = shift;
    my $name = shift;
    my $module = shift;
    my $spawn_name = $self->{prefix} . $name;
    die "Installation $spawn_name does not exist" unless $self->{spawns}{$spawn_name};
    die "Command pipe for $module already exists" if $self->{pipes}{$module};
    $self->{pipes}{$module} = $self->{spawns}{$spawn_name}->cmdpipe($module);
    return 1;
  }

sub log
  {
    my $self = shift;
    my $name = shift;
    my $spawn_name = $self->{prefix} . $name;
    die "Installation $spawn_name does not exist" unless $self->{spawns}{$spawn_name};
    return $self->{spawns}{$spawn_name}->log(@_);
  }

1;
