# Copyright (C) 2002 Andrew Suffield <asuffield@users.sourceforge.net>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA

package Dancer::CmdScript::Call;

use strict;
use 5.006;
use warnings;

use Dancer::CmdInterpreter;

BEGIN
  {
    use Exporter;
    our @ISA = qw/Exporter/;
    our $VERSION = "0.0.1";
    our @EXPORT = qw//;
    our @EXPORT_OK = qw//;
  }

sub define
  {
    my $script = shift;
    my $line = shift;
    die "Insufficient arguments to define" unless scalar @_ >= 2;
    my $name = (shift)->eval($script);
    my $block = shift;

    $script->data()->{__PACKAGE__}{$name} = $block;
  }

sub call
  {
    my $script = shift;
    my $line = shift;
    die "Insufficient arguments to call" unless scalar @_ >= 1;
    my $name = (shift)->eval($script);

    my $block = $script->data()->{__PACKAGE__}{$name};
    die "Undefined function $name" unless defined $block;

    my $i = 1;
    while (my $arg = shift)
      {
        $script->variables()->{$i++} = $arg->eval($script);
      }

    my $ret = $block->eval($script, $name, $line);

    delete $script->variables()->{$_} foreach 1..($i - 1);

    return $ret;
  }

Dancer::CmdInterpreter::register_function('define', \&define);
Dancer::CmdInterpreter::register_function('call', \&call);

1;
