# Copyright (C) 2002 Andrew Suffield <asuffield@users.sourceforge.net>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA

package Dancer::CmdScript::Conditional;

use strict;
use 5.006;
use warnings;

use Dancer::CmdScript::Block;

BEGIN
  {
    use Exporter;
    our @ISA = qw/Exporter/;
    our $VERSION = "0.0.1";
    our @EXPORT = qw//;
    our @EXPORT_OK = qw//;
  }

sub new
  {
    my $class = shift;
    my ($line, $cond, $then, $else) = @_;

    my $self = {
                line => $line,
                cond => $cond,
                then => $then || Dancer::CmdScript::Block->new($line),
                else => $else || Dancer::CmdScript::Block->new($line),
               };
    bless $self, $class;
    return $self;
  }

sub lines
  {
    my $self = shift;
    return scalar @{$self->{lines}};
  }

sub interpolate
  {
    my $self = shift;
    my $interp = shift;
    $_->interpolate($interp) foreach $self->{cond}, $self->{then}, $self->{else};
  }

sub optimise
  {
    my $self = shift;
    $self->{cond} = $self->{cond}->optimise();

    # We can optimise out an if entirely in the case where the condition is constant
    if ($self->{cond}->constant())
      {
        if ($self->{cond}->eval())
          {
            return $self->{then}->optimise();
          }
        else
          {
            return $self->{else}->optimise();
          }
      }
    else
      {
        # Otherwise, optimise them both
        $self->{then} = $self->{then}->optimise();
        $self->{else} = $self->{else}->optimise();
        # If they are both constant and equal, return one of them as our value
        if ($self->{then}->constant() and $self->{else}->constant())
          {
            return $self->{then} if $self->{then}->eval() eq $self->{else}->eval();
          }
      }
    return $self;
  }

sub constant
  {
    # There are no interesting constant cases that optimise cannot flatten out anyway
    return 0;
  }

sub eval
  {
    my $self = shift;
    my $script = shift;

    if ($self->{cond}->eval($script))
      {
        return $self->{then}->eval($script);
      }
    else
      {
        return $self->{else}->eval($script);
      }
  }

package Dancer::CmdScript::If;
our @ISA = qw/Dancer::CmdScript::Conditional/;

1;
