# Copyright (C) 2002 Andrew Suffield <asuffield@users.sourceforge.net>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA

package Dancer::CmdScript::Expression::BinOp;

use strict;
use 5.006;
use warnings;

BEGIN
  {
    use Exporter;
    our @ISA = qw/Exporter/;
    our $VERSION = "0.0.1";
    our @EXPORT = qw//;
    our @EXPORT_OK = qw//;
  }

sub new
  {
    my $class = shift;
    my ($line, $a, $b) = @_;

    my $self = {
                line => $line,
                a => $a,
                b => $b,
               };
    bless $self, $class;
    return $self;
  }

sub interpolate
  {
    my $self = shift;
    my $interp = shift;
    $self->{a}->interpolate($interp);
    $self->{b}->interpolate($interp);
  }

sub optimise
  {
    my $self = shift;
    $self->{a} = $self->{a}->optimise();
    $self->{b} = $self->{b}->optimise();
    if ($self->{a}->constant() and $self->{b}->constant())
      {
        return new Dancer::CmdScript::Expression::Value $self->{line}, $self->eval();
      }
    return $self;
  }

sub constant
  {
    my $self = shift;
    return $self->{a}->constant() and $self->{b}->constant();
  }

my %binops = (Equals => '==',
              NotEquals => '!=',
              LessThan => '<',
              LessThanOrEquals => '<=',
              GreaterThan => '>',
              GreaterThanOrEquals => '>=',
              StrEquals => 'eq',
              StrNotEquals => 'ne',
              StrLessThan => 'lt',
              StrLessThanOrEquals => 'le',
              StrGreaterThan => 'gt',
              StrGreaterThanOrEquals => 'ge',
              Add => '+',
              Subtract => '-',
              Product => '*',
              Divide => '/',
              Concatenate => '.',
             );

foreach (keys %binops)
  {
    my $name = $_;
    my $sign = $binops{$name};
    my $func = 'package Dancer::CmdScript::Expression::' . $name . ";\n";
    $func .= <<'END';
our @ISA = qw/Dancer::CmdScript::Expression::BinOp/;

sub eval
  {
    my $self = shift;
    my $script = shift;

    return $self->{a}->eval($script)
END
    $func .= $sign . '$self->{b}->eval($script);}';
    eval $func;
  }

1;
