# Copyright (C) 2002 Andrew Suffield <asuffield@users.sourceforge.net>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA

package Dancer::CmdScript::Function;

use strict;
use 5.006;
use warnings;

BEGIN
  {
    use Exporter;
    our @ISA = qw/Exporter/;
    our $VERSION = "0.0.1";
    our @EXPORT = qw//;
    our @EXPORT_OK = qw//;
  }

sub new
  {
    my $class = shift;
    my ($line, $name, $args) = @_;

    my $self = {
                line => $line,
                name => $name,
                sub => undef,
                runtime_args => $args || [],
                compiletime_args => undef,
               };
    bless $self, $class;
    return $self;
  }

sub interpolate
  {
    my $self = shift;
    my $interpreter = shift;

    my $func = $interpreter->function($self->{name});
    die {line => $self->{line}, msg => "Function $self->{name} is not defined"} unless $func;
    $self->{sub} = $func->{sub};
    $self->{compiletime_args} = $func->{args};

    $_->interpolate($interpreter) foreach @{$self->{runtime_args}};
  }

sub optimise
  {
    my $self = shift;
    for (my $i = 0; $i < scalar @{$self->{runtime_args}}; $i++)
      {
        $self->{runtime_args}->[$i] = $self->{runtime_args}->[$i]->optimise();
      }
    return $self;
  }

sub constant
  {
    return 0;
  }

sub eval
  {
    my $self = shift;
    my $script = shift;

    my $ret;
    eval
      {
        $ret = $self->{sub}->($script, $self->{line}, @{$self->{compiletime_args}}, @{$self->{runtime_args}});
      };
    if ($@)
      {
        # Hashify it if it isn't already a hash
        die {line => $self->{line}, msg => $@} unless ref $@ eq 'HASH';
        # Otherwise just punt it
        die $@;
      }
    return $ret;
  }

1;
