# Copyright (C) 2002 Andrew Suffield <asuffield@users.sourceforge.net>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA

package Dancer::CmdScript::Function::Spawn;

use strict;
use 5.006;
use warnings;

use Dancer::CmdInterpreter;
use Dancer::Spawn;

BEGIN
  {
    use Exporter;
    our @ISA = qw/Exporter/;
    our $VERSION = "0.0.1";
    our @EXPORT = qw//;
    our @EXPORT_OK = qw//;
  }

sub preinstall
  {
    my $script = shift;
    my $line = shift;
    my $name = shift or die "No spawn name given";
    $name = $script->prefix() . $name->eval($script);
    die "Installation $name already exists" if $script->spawns()->{$name};
    my $spawn = $script->spawns()->{$name} = new Dancer::Spawn $name;
    $spawn->install();
    return 1;
  }

sub spawn
  {
    my $script = shift;
    my $line = shift;
    my $name = shift or die "No spawn name given";
    $name = $script->prefix() . $name->eval($script);
    my $spawn = $script->spawns()->{$name} || new Dancer::Spawn $name;;
    die "$name is already spawned" if $spawn->spawned();

    my @modules = ();
    my @args = ();
    my $ref;
    while (my $arg = shift)
      {
        $arg = $arg->eval($script);
        if ($arg eq 'modules')
          {
            $ref = \@modules;
            next;
          }
        elsif ($arg eq 'args')
          {
            $ref = \@args;
            next;
          }
        else
          {
            die "Invalid argument $arg to spawn" unless $ref;
            push @{$ref}, $arg;
          }
      }

    eval
      {
        $script->spawns()->{$name} = $spawn;
        $spawn->install() unless $spawn->installed();
        $spawn->start(modules => \@modules, args => \@args);
      };
    if ($@)
      {
        delete $script->spawns()->{$name};
        die $@;
      }
    return 1;
  }

Dancer::CmdInterpreter::register_function('preinstall', \&preinstall);
Dancer::CmdInterpreter::register_function('spawn', \&spawn);

1;
