# Copyright (C) 2002 Andrew Suffield <asuffield@users.sourceforge.net>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA

package Dancer::CmdScript::IO;

use strict;
use 5.006;
use warnings;

BEGIN
  {
    use Exporter;
    our @ISA = qw/Exporter/;
    our $VERSION = "0.0.1";
    our @EXPORT = qw//;
    our @EXPORT_OK = qw//;
  }

sub new
  {
    my $class = shift;
    my ($line, $connection, $regexp, $expr) = @_;

    my $self = {
                line => $line,
                connection => $connection,
                regexp => $regexp,
                expr => $expr,
               };
    bless $self, $class;
    return $self;
  }

sub interpolate
  {
    return;
  }

sub optimise
  {
    my $self = shift;
    return $self;
  }

# IO operations aren't constant
sub constant
  {
    return 0;
  }

package Dancer::CmdScript::Get;
our @ISA = qw/Dancer::CmdScript::IO/;

sub eval
  {
    my $self = shift;
    my $script = shift;

    my $connection = $script->connections()->{$self->{connection}};
    die {line => $self->{line}, msg => "Connection $self->{connection} does not exist"} unless $connection;
    my $res = eval {$connection->get($self->{line})};
    die {line => $self->{line}, msg => $@} if $@;
    return $res;
  }

package Dancer::CmdScript::Read;
our @ISA = qw/Dancer::CmdScript::IO/;

sub eval
  {
    my $self = shift;
    my $script = shift;

    my $connection = $script->connections()->{$self->{connection}};
    die {line => $self->{line}, msg => "Connection $self->{connection} does not exist"} unless $connection;
    my $res = eval {$connection->read($self->{line}, $self->{regexp},
                                      $self->{regexp} ?
                                      $script->interpolate_regexp($self->{expr}->eval($script)) :
                                      $script->interpolate_string($self->{expr}->eval($script))
                                     )};
    if ($@)
      {
        $script->variables()->{'_'} = undef;
        die {line => $self->{line}, msg => $@};
      }
    $script->variables()->{'_'} = $res;
  }

package Dancer::CmdScript::Write;
our @ISA = qw/Dancer::CmdScript::IO/;

sub eval
  {
    my $self = shift;
    my $script = shift;

    my $connection = $script->connections()->{$self->{connection}};
    die {line => $self->{line}, msg => "Connection $self->{connection} does not exist"} unless $connection;
    eval {$connection->write($self->{line}, $script->interpolate_string($self->{expr}->eval($script)))};
    die {line => $self->{line}, msg => $@} if $@;
  }

package Dancer::CmdScript::Open;

sub new
  {
    my $class = shift;
    my ($line, $name, $stereotype, $data) = @_;

    my $self = {
                line => $line,
                name => $name,
                stereotype => $stereotype,
                data => $data,
               };
    bless $self, $class;
    return $self;
  }

sub interpolate
  {
    my $self = shift;
    my $interp = shift;
    my $stereo = $interp->stereotype($self->{stereotype});
    die {line => $self->{line}, msg => "Unknown stereotype $self->{stereotype}"} unless $stereo;
    $self->{sub} = $stereo->{open};
  }

sub optimise
  {
    my $self = shift;
    return $self;
  }

sub constant
  {
    return 0;
  }

sub eval
  {
    my $self = shift;
    my $script = shift;

    die {line => $self->{line}, msg => "Connection $self->{name} already exists"}
      if $script->connections()->{$self->{name}};
    my $connection = $self->{sub}->($script, $self->{name},
                                    map {$script->interpolate_string($_->eval($script))} @{$self->{data}});
    $script->connections()->{$self->{name}} = $connection;
    return 1;
  }

package Dancer::CmdScript::Listen;
our @ISA = qw/Dancer::CmdScript::Open/;

sub interpolate
  {
    my $self = shift;
    my $interp = shift;
    my $stereo = $interp->stereotype($self->{stereotype});
    die {line => $self->{line}, msg => "Unknown stereotype $self->{stereotype}"} unless $stereo;
    $self->{sub} = $stereo->{listen};
    die "Listening not supported by stereotype $self->{stereotype}" unless $self->{sub};
  }

package Dancer::CmdScript::Close;

sub new
  {
    my $class = shift;
    my ($line, $name) = @_;

    my $self = {
                line => $line,
                connection => $name,
               };
    bless $self, $class;
    return $self;
  }

sub interpolate
  {
    return;
  }

sub optimise
  {
    my $self = shift;
    return $self;
  }

sub constant
  {
    return 0;
  }

sub eval
  {
    my $self = shift;
    my $script = shift;

    my $connection = $script->connections()->{$self->{connection}};
    die {line => $self->{line}, msg => "Connection $self->{connection} does not exist"} unless $connection;
    $connection->close();
    delete $script->connections()->{$self->{connection}};
    return 1;
  }

package Dancer::CmdScript::EOF;
our @ISA = qw/Dancer::CmdScript::Close/;

sub eval
  {
    my $self = shift;
    my $script = shift;

    my $connection = $script->connections()->{$self->{connection}};
    die {line => $self->{line}, msg => "Connection $self->{connection} does not exist"} unless $connection;
    eval {$connection->eof($self->{line});};
    die {line => $self->{line}, msg => $@} if $@;
    delete $script->connections()->{$self->{connection}};
    return 1;
  }

package Dancer::CmdScript::Accept;
our @ISA = qw/Dancer::CmdScript::Close/;

sub eval
  {
    my $self = shift;
    my $script = shift;

    my $connection = $script->connections()->{$self->{connection}};
    die {line => $self->{line}, msg => "Connection $self->{connection} does not exist"} unless $connection;
    eval {$connection->accept($self->{line});};
    die {line => $self->{line}, msg => $@} if $@;
    return 1;
  }

1;
