# Copyright (C) 2002 Andrew Suffield <asuffield@users.sourceforge.net>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA

package Dancer::Message;

use strict;
use 5.006;
use warnings;

use Carp;
use overload '""' => \&stringify;

BEGIN
  {
    use Exporter;
    our @ISA = qw/Exporter/;
    our $VERSION = "0.0.1";
    our @EXPORT = qw//;
    our @EXPORT_OK = qw/check_message/;
  }

sub new($%)
  {
    my $class = shift;
    my %args = @_;

    $args{type} ||= "**INVALID**";
    $args{origin} ||= undef;
    $args{dest} ||= undef;
    $args{str} ||= [];
    $args{uint32} ||= [];
    $args{sint32} ||= [];

    my $self = \%args;
    bless $self, $class;
    return $self;
  }

sub build($)
  {
    my $self = shift;
    my @str;
    push @str, '"' . $self->{type} . '"';
    push @str, ':';
    push @str, '<' . $self->{origin} if defined $self->{origin};
    push @str, '>' . $self->{dest} if defined $self->{dest};
    push @str, map {defined $_ ? "\"$_\"" : "(null)"} @{$self->{str}};
    push @str, @{$self->{uint32}};
    push @str, map {sprintf "%+d", $_} @{$self->{sint32}};
    return join ' ', @str;
  }

sub stringify
  {
    my $self = shift;
    my ($a, $b) = @_;
    $a ||= "";
    $b ||= "";
    return $a . $self->build() . $b;
  }

sub _do_check($$)
  {
    my $self = shift;
    my $pipe = shift;
    my $caller = shift;
    my $i;
    $pipe->check_cmd_value("GET_MSG_TYPE", $self->{type}, "check message type", undef, $caller);
    $pipe->check_cmd_value("GET_ORIGIN", defined $self->{origin} ? $self->{origin} : "(null)", "check message origin",
                           undef, $caller);
    $pipe->check_cmd_value("GET_DEST", defined $self->{dest} ? $self->{dest} : "(null)", "check message dest",
                           undef, $caller);
    foreach my $type (qw/str uint32 sint32/)
      {
        my $uctype = uc $type;
        $pipe->check_cmd_value("GET_${uctype}C", scalar @{$self->{$type}}, "check $type count",
                               undef, $caller) or return 0;
        $i = 0;
        foreach (@{$self->{$type}})
          {
            $_ = "(null)" unless defined $_;
            $pipe->check_cmd_value("GET_${uctype}V $i", $_, "check $type $i", undef, $caller);
            $i++;
          }
      }
    return 1;
  }

sub check($$;$)
  {
    my ($self, $pipe, $caller) = @_;
    $pipe->check_cmd_value("PEEK_MESSAGE", "YES", "messages are waiting", undef, $caller || [caller]) or return 0;
    $self->_do_check($pipe, $caller || [caller]);
    $pipe->do("POP_MESSAGE", "pop message", undef, $caller || [caller]);
  }

sub send($$$;$)
  {
    my ($self, $pipe, $conn, $caller) = @_;
    $pipe->do("SEND_MESSAGE $conn $self", "send message to session $conn", undef, $caller || [caller]);
  }

sub check_message($$%)
  {
    my $pipe = shift;
    my $conn = shift;
    (new Dancer::Message @_)->check($pipe, [caller]);
  }

1;
