# Copyright (C) 2002 Andrew Suffield <asuffield@users.sourceforge.net>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA

package DejaPerl::Interpreter::CmdScript;

use strict;
use 5.006;
use warnings;

use Carp;
use DejaPerl qw/:framework :interpreter/;
use Dancer::CmdInterpreter;

BEGIN
  {
    use Exporter;
    our @ISA = qw/Exporter/;
    our $VERSION = "0.0.1";
    our @EXPORT = qw//;
  }

my %loaded = ();

sub load
  {
    return 0;
  }

my $interp = new Dancer::CmdInterpreter or die "Failed to create interpreter";
my $script;

sub runtest
  {
    my $test = shift;
    $script = $interp->compile_script($test) or die "Failed to load $test";
    $script->run();
    $script = undef;
    return 1;
  }

sub stack
  {
    my $caller = shift;
    my @stack;
    my $current_line = $caller->[2];
    my $next_function = $caller->[1];
    foreach (@{$script->stack()})
      {
        unshift @stack, [undef, $next_function, $_->[2]];
        $next_function = $_->[1];
      }
    unshift @stack, [undef, $next_function, $current_line];
    return "(" . join(" called from ", map {"$_->[1]:$_->[2]"} @stack) . ")";
  }

register_interpreter(name => 'CmdScript',
                     extensions => [qw/cs/],
                     load => \&load,
                     runtest => \&runtest,
                     stack => \&stack,
                     priority => 50,
                    );

1;
