# Copyright (C) 2002 Andrew Suffield <asuffield@users.sourceforge.net>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA

use strict;
use warnings;
use 5.006;

use Carp;
use DejaPerl ':framework';
use DejaPerl::Interpreter::CmdScript;
use Dancer::CmdScript::Function::Config;
use Dancer::CmdScript::Function::Debug;
use Dancer::CmdScript::Function::Eval;
use Dancer::CmdScript::Function::Kill;
use Dancer::CmdScript::Function::Sleep;
use Dancer::CmdScript::Function::Spawn;
use Dancer::CmdScript::Function::SpawnPath;
use Dancer::CmdScript::Function::System;
use Dancer::CmdScript::Stereotype::Log;
use Dancer::CmdScript::Stereotype::CmdPipe;
use Dancer::CmdScript::Stereotype::CmdPipe::Message;
use Dancer::CmdScript::Stereotype::DGMTP;
use Dancer::CmdScript::Stereotype::RFC1459;
use Dancer::CmdScript::Stereotype::Socket;
use Expect;
$Expect::Multiline_Matching = 0;

$TOOL_EXECUTABLE ||= find_file("dancerd", "$objdir/../main", "/usr/local/sbin", "/usr/sbin");
confess "Failed to find a dancerd binary, specify with --tool_exec" unless $TOOL_EXECUTABLE;

my $default_timeout = 5;

my $work_dir = "$objdir/test-work";
my $install_tmp_dir = "$work_dir/install-tmp";
if (-e $install_tmp_dir and not $ENV{DANCER_SKIP_INSTALL})
  {
    verbose "Removing old tree $install_tmp_dir...";
    rmtree($install_tmp_dir, 0, 1);
    die "Couldn't remove tree $install_tmp_dir" if -e $install_tmp_dir;
  }

sub dancer_version()
  {
    verbose "Invoking dancerd to get version...", 2;
    my $version = `$TOOL_EXECUTABLE -V` or confess "Failed to run dancerd -V";
    my $path = `which "$TOOL_EXECUTABLE"` or confess "Failed to find full path of $TOOL_EXECUTABLE";
    $version =~ /^dancer (\S+)(?: \((.*)\))?/ or confess "Couldn't parse output of dancerd -V";
    verbose "Dancer is version $1 build-id $2", 2;
    return "$path: version $1 build-id $2";
  }

sub check_expect($$$;$)
  {
    my $exp = shift;
    my $re = shift;
    my $description = shift;
    my $timeout = shift || $default_timeout;
    my $got = 0;
    my (undef, $err, undef, undef, undef) =
      $exp->expect($timeout, -re => $re, sub {pass "got $description"; $got = 1});
    if ($err)
      {
        if ($err =~ /TIMEOUT/)
          {
            fail "no $description (timeout)";
          }
        elsif ($err =~ /EOF/)
          {
            fail "no $description (eof)";
          }
        elsif ($err =~ s/^3://)
          {
            return 1 if $got;
            error $err;
          }
        else
          {
            error "Unknown error: $err (\$! eq $!)";
          }
        return 0;
      }
    confess "Not reached" unless $got;
    return 1;
  }

1;
