/************************************************************************
 *   This is part of Dancer, a text relay daemon
 *   Copyright (C) 2002 Andrew Suffield <asuffield@users.sourceforge.net>
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 */

#include <dancerd/config.h>
#include <dancerd/modules.h>
#include <dancerd/log.h>
#include <dancerd/xmalloc.h>
#include <dancerd/test/cmdpipe.h>
#include <dancerd/config/items.h>

#include <stddef.h>
#include <string.h>
#include <stdio.h>

SETUP_MODULE("test.config", MODVERSION);

IMPORTED_SYMBOL(config, get_item);
IMPORTED_SYMBOL(config, put_item);
IMPORTED_SYMBOL(config, delete_item);

#include "symtab"

static struct cmdpipe *cp = NULL;

static int
parse_command(char *data, size_t len unused_attribute)
{
  struct config_item item;

  if (strncasecmp(data, "GET", strlen("GET")) == 0)
    {
      char *c = data + strlen("GET");
      if (*c)
	c++;
      if (SYM(config, get_item)(c, &item))
	{
	  switch(item.type)
	    {
	    case citem_bool:
	      if (item.boolean)
		print_output_buffer(cp, "true\n");
	      else
		print_output_buffer(cp, "false\n");
	      break;
	    case citem_string:
	      print_output_buffer(cp, "\"");
	      print_output_buffer(cp, item.str);
	      print_output_buffer(cp, "\"\n");
	      break;
	    case citem_uint32:
	      {
		char buf[512];
		long unsigned int i = item.uint32;
		snprintf(buf, 512, "%lu", i);
		print_output_buffer(cp, buf);
		print_output_buffer(cp, "\n");
		break;
	      }
	    case citem_sint32:
	      {
		char buf[512];
		long signed int i = item.sint32;
		snprintf(buf, 512, "%+ld", i);
		print_output_buffer(cp, buf);
		print_output_buffer(cp, "\n");
		break;
	      }
	    case citem_null:
	      print_output_buffer(cp, "null\n");
              break;
            default:
              return 0;
	    }
	  return 1;
	}
      else
	{
	  return 0;
	}
    }
  else if (strncasecmp(data, "PUT", strlen("PUT")) == 0)
    {
      char *c = data + strlen("PUT"), *d = NULL;
      if (*c)
	{
	  c++;
	  d = strchr(c, ' ');
	}
      if (!d)
	return -1;
      else
	{
	  item.key = c;
	  *d++ = '\0';
	  switch (*d)
	    {
	    case 't':
	      item.type = citem_bool;
	      item.boolean = 1;
	      break;
	    case 'f':
	      item.type = citem_bool;
	      item.boolean = 0;
	      break;
	    case '"':
	      item.type = citem_string;
	      item.str = d + 1;
	      if ((d = strchr(item.str, '"')))
		*d = '\0';
	      else
		{
		  return 0;
		}
	      break;
	    case '+':
	    case '-':
	      item.type = citem_sint32;
	      item.sint32 = strtol(d, NULL, 0);
	      break;
	    case '0': case '1': case '2': case '3': case '4':
	    case '5': case '6': case '7': case '8': case '9':
	      item.type = citem_uint32;
	      item.uint32 = strtoul(d, NULL, 0);
	      break;
	    default:
	      return 0;
	    }
	  if (SYM(config, put_item)(&item))
	    return 1;
	  else
	    return 0;
	}
    }
  else if (strncasecmp(data, "DELETE", strlen("DELETE")) == 0)
    {
      char *c = data + strlen("DELETE");
      if (*c)
	c++;
      if (SYM(config, delete_item)(c))
	return 1;
      else
	return 0;
    }
  else
    return 0;
}

static void
pipe_fail(void)
{
  unload_module(MYNAME);
}

int
MODULE_INIT(void *data unused_attribute)
{
  if (!(cp = setup_cmdpipe(MYHANDLE, MYNAME, 32768, &parse_command, &pipe_fail)))
    return 0;

  return MODULE_LOAD_OK;
}

int
MODULE_DEINIT(const char *new_version unused_attribute, module_init_func *init unused_attribute)
{
  if (cp)
    close_cmdpipe(cp);
  cp = NULL;
  return MODULE_UNLOAD_OK;
}
