/************************************************************************
 *   This is part of Dancer, a text relay daemon
 *   Copyright (C) 2002 Andrew Suffield <asuffield@users.sourceforge.net>
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 */

#include <dancerd/config.h>
#include <dancerd/modules.h>
#include <dancerd/log.h>
#include <dancerd/xmalloc.h>
#include <dancerd/dbuffer.h>
#include <dancerd/test/cmdpipe.h>

#include <stddef.h>
#include <string.h>
#include <stdio.h>

SETUP_MODULE("test.main.dbuffer", MODVERSION);

static struct cmdpipe *cp = NULL;

static int
parse_command(char *data, size_t len unused_attribute)
{
  static struct dbuffer *buf = NULL;

  if (strncasecmp(data, "NEW_DBUFFER", strlen("NEW_DBUFFER")) == 0)
    {
      buf = new_dbuffer();
      return 1;
    }
  else if (strncasecmp(data, "DESTROY_DBUFFER", strlen("DESTROY_DBUFFER")) == 0)
    {
      if (!buf)
	return -1;
      else
	{
	  free_dbuffer(buf);
	  buf = NULL;
	  return 1;
	}
    }
  else if (strncasecmp(data, "ADD", strlen("ADD")) == 0)
    {
      char *c = data + strlen("ADD");
      if (*c)
	c++;
      if (!buf)
	return -1;
      else
	{
	  dbuffer_add(buf, c, strlen(c));
	  return 1;
	}
    }
  else if (strncasecmp(data, "GET", strlen("GET")) == 0)
    {
      char *c = data + strlen("GET");
      if (*c)
	c++;
      if (!buf)
	return -1;
      else
	{
	  size_t slen = strtoul(c, NULL, 0);
	  char b[16384];
	  slen = dbuffer_get(buf, b, (slen < 16383) ? slen : 16383);
	  b[(slen < 16383) ? slen : 16383] = '\0';
	  print_output_buffer(cp, b);
	  print_output_buffer(cp, "\n");
	  return 1;
	}
    }
  else if (strncasecmp(data, "REMOVE", strlen("REMOVE")) == 0)
    {
      char *c = data + strlen("REMOVE");
      if (*c)
	c++;
      if (!buf)
	return -1;
      else
	{
	  size_t slen = strtoul(c, NULL, 0);
	  dbuffer_remove(buf, slen);
	  return 1;
	}
    }
  else if (strncasecmp(data, "EXTRACT", strlen("EXTRACT")) == 0)
    {
      char *c = data + strlen("EXTRACT");
      if (*c)
	c++;
      if (!buf)
	return -1;
      else
	{
	  size_t slen = strtoul(c, NULL, 0);
	  char b[16384];
	  slen = dbuffer_extract(buf, b, (slen < 16383) ? slen : 16383);
	  b[(slen < 16383) ? slen : 16383] = '\0';
	  print_output_buffer(cp, b);
	  print_output_buffer(cp, "\n");
	  return 1;
	}
    }
  else if (strncasecmp(data, "FIND", strlen("FIND")) == 0)
    {
      char *c = data + strlen("FIND");
      if (*c)
	c++;
      if (!buf)
	return -1;
      else
	{
	  char b[64];
	  ssize_t pos = dbuffer_find_char(buf, *c);
	  snprintf(b, 64, "%zd\n", pos);
	  print_output_buffer(cp, b);
	  return 1;
	}
    }
  else if (strncasecmp(data, "LENGTH", strlen("LENGTH")) == 0)
    {
      if (!buf)
	return -1;
      else
	{
	  char b[64];
	  ssize_t pos = dbuffer_len(buf);
	  snprintf(b, 64, "%zd\n", pos);
	  print_output_buffer(cp, b);
	  return 1;
	}
    }
  else if (strncasecmp(data, "SANITY", strlen("SANITY")) == 0)
    {
      if (dbuffer_check_sanity(buf))
	print_output_buffer(cp, "OK\n");
      else
	print_output_buffer(cp, "ERROR\n");
      return 1;
    }
  else
    return 0;
}

static void
pipe_fail(void)
{
  unload_module(MYNAME);
}

int
MODULE_INIT(void *data unused_attribute)
{
  if (!(cp = setup_cmdpipe(MYHANDLE, MYNAME, 65536 * 128, &parse_command, &pipe_fail)))
    return 0;

  return MODULE_LOAD_OK;
}

int
MODULE_DEINIT(const char *new_version unused_attribute, module_init_func *init unused_attribute)
{
  if (cp)
    close_cmdpipe(cp);
  cp = NULL;
  return MODULE_UNLOAD_OK;
}
