/************************************************************************
 *   This is part of Dancer, a text relay daemon
 *   Copyright (C) 2002 Andrew Suffield <asuffield@users.sourceforge.net>
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 */

#include <dancerd/config.h>
#include <dancerd/modules.h>
#include <dancerd/log.h>
#include <dancerd/xmalloc.h>
#include <dancerd/xstring.h>
#include <dancerd/hashtable.h>
#include <dancerd/test/cmdpipe.h>

#include <stddef.h>
#include <string.h>

SETUP_MODULE("test.main.hashtable", MODVERSION);

static struct cmdpipe *cp = NULL;

static int
parse_command(char *data, size_t len unused_attribute)
{
  static struct hashtable *test_table = NULL;

  if (strncasecmp(data, "NEW_HASHTABLE", strlen("NEW_HASHTABLE")) == 0)
    {
      test_table = new_hashtable(&hash_ascii, 23);
      return 1;
    }
  else if (strncasecmp(data, "DESTROY_HASHTABLE", strlen("DESTROY_HASHTABLE")) == 0)
    {
      if (!test_table)
	return -1;
      else
	{
          free_hashtable(test_table);
	  test_table = NULL;
	  return 1;
	}
    }
  else if (strncasecmp(data, "INSERT", strlen("INSERT")) == 0)
    {
      char *c = data + strlen("INSERT"), *d = NULL;;
      if (*c)
        {
          c++;
	  d = strchr(c, ' ');
        }
      if (!test_table || !d)
	return -1;
      else
	{
          *d++ = '\0';
	  hashtable_insert(test_table, c, xstrdup(d));
	  return 1;
	}
    }
  else if (strncasecmp(data, "LOOKUP", strlen("LOOKUP")) == 0)
    {
      char *c = data + strlen("LOOKUP");
      if (c)
        c++;
      if (!test_table)
	return -1;
      else
	{
          void *d = hashtable_lookup(test_table, c);
          if (d)
            print_output_buffer(cp, d);
          else
            print_output_buffer(cp, "(null)");
	  print_output_buffer(cp, "\n");
	  return 1;
	}
    }
  else if (strncasecmp(data, "DELETE", strlen("DELETE")) == 0)
    {
      char *c = data + strlen("LOOKUP");
      if (c)
        c++;
      if (!test_table)
	return -1;
      else
	{
          xfree(hashtable_lookup(test_table, c));
          hashtable_delete(test_table, c);
	  return 1;
	}
    }
  else if (strncasecmp(data, "SANITY", strlen("SANITY")) == 0)
    {
      if (hashtable_check_sanity(test_table))
	print_output_buffer(cp, "OK\n");
      else
	print_output_buffer(cp, "ERROR\n");
      return 1;
    }
  else
    return 0;
}

static void
pipe_fail(void)
{
  unload_module(MYNAME);
}

int
MODULE_INIT(void *data unused_attribute)
{
  if (!(cp = setup_cmdpipe(MYHANDLE, MYNAME, 32768, &parse_command, &pipe_fail)))
    return 0;

  return MODULE_LOAD_OK;
}

int
MODULE_DEINIT(const char *new_version unused_attribute, module_init_func *init unused_attribute)
{
  if (cp)
    close_cmdpipe(cp);
  cp = NULL;
  return MODULE_UNLOAD_OK;
}
