/************************************************************************
 *   This is part of Dancer, a text relay daemon
 *   Copyright (C) 2002 Andrew Suffield <asuffield@users.sourceforge.net>
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 */

#include <dancerd/config.h>
#include <dancerd/modules.h>
#include <dancerd/log.h>
#include <dancerd/xstring.h>
#include <dancerd/test/cmdpipe.h>
#include <dancerd/version.h>

#include <stddef.h>
#include <stdio.h>

SETUP_MODULE("test.main", MODVERSION);

static struct cmdpipe *cp = NULL;

static int
parse_command(char *data, size_t len unused_attribute)
{
  if (strncasecmp(data, "XSTRVERSCMP", strlen("XSTRVERSCMP")) == 0)
    {
      char *c = data + strlen("XSTRVERSCMP"), *d = NULL;
      if (*c)
        {
          c++;
	  d = strchr(c, ' ');
        }
      if (!d)
	return -1;
      else
	{
          int x;
          *d++ = '\0';
          x = xstrverscmp(c, d);
          if (x < 0)
            print_output_buffer(cp, "<\n");
          else if (x == 0)
            print_output_buffer(cp, "==\n");
          else
            print_output_buffer(cp, ">\n");
	  return 1;
	}
    }
  else if (strncasecmp(data, "STRIP_SPACES", strlen("STRIP_SPACES")) == 0)
    {
      char *c = data + strlen("STRIP_SPACES");
      if (*c)
        c++;
      if (!c)
	return -1;
      else
        {
          print_output_buffer(cp, strip_spaces(c));
          print_output_buffer(cp, "\n");
          return 1;
        }
    }
  else if (strncasecmp(data, "STRIP_CONTROL", strlen("STRIP_CONTROL")) == 0)
    {
      char *c = data + strlen("STRIP_CONTROL");
      if (*c)
        c++;
      if (!c)
	return -1;
      else
        {
          print_output_buffer(cp, strip_control(c));
          print_output_buffer(cp, "\n");
          return 1;
        }
    }
  else if (strncasecmp(data, "GET_VERSION", strlen("GET_VERSION")) == 0)
    {
      print_output_buffer(cp, dancer_version);
      print_output_buffer(cp, "\n");
      return 1;
    }
  else if (strncasecmp(data, "GET_SERIAL", strlen("GET_SERIAL")) == 0)
    {
      print_output_buffer(cp, serial_number);
      print_output_buffer(cp, "\n");
      return 1;
    }
  else if (strncasecmp(data, "GET_BUILD_PLATFORM", strlen("GET_BUILD_PLATFORM")) == 0)
    {
      print_output_buffer(cp, build_platform);
      print_output_buffer(cp, "\n");
      return 1;
    }
  else if (strncasecmp(data, "GET_BUILD_TIME", strlen("GET_BUILD_TIME")) == 0)
    {
      print_output_buffer(cp, build_time);
      print_output_buffer(cp, "\n");
      return 1;
    }
  else
    return 0;
}

static void
pipe_fail(void)
{
  unload_module(MYNAME);
}

int
MODULE_INIT(void *data unused_attribute)
{
  if (!(cp = setup_cmdpipe(MYHANDLE, MYNAME, 32768, &parse_command, &pipe_fail)))
    return 0;

  return MODULE_LOAD_OK;
}

int
MODULE_DEINIT(const char *new_version unused_attribute, module_init_func *init unused_attribute)
{
  if (cp)
    close_cmdpipe(cp);
  cp = NULL;
  return MODULE_UNLOAD_OK;
}
