/************************************************************************
 *   This is part of Dancer, a text relay daemon
 *   Copyright (C) 2002 Andrew Suffield <asuffield@users.sourceforge.net>
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 */

#include <dancerd/config.h>
#include <dancerd/modules.h>
#include <dancerd/log.h>
#include <dancerd/xmalloc.h>
#include <dancerd/xstring.h>
#include <dancerd/modules.h>
#include <dancerd/test/cmdpipe.h>

#include <stddef.h>
#include <string.h>

SETUP_MODULE("test.main.modules", MODVERSION);

static struct cmdpipe *cp = NULL;

static void
unload_handler(void *d)
{
  char *c = d;
  print_output_buffer(cp, "UNLOAD: ");
  print_output_buffer(cp, c);
  print_output_buffer(cp, "\n**DONE**\n");
  xfree(c);
}

static int
parse_command(char *data, size_t len unused_attribute)
{
  if (strncasecmp(data, "FIND_MODULE", strlen("FIND_MODULE")) == 0)
    {
      char *c = data + strlen("FIND_MODULE");
      if (c)
        c++;
      if (!c)
	return -1;
      else
	{
          if (find_module(c))
            print_output_buffer(cp, "YES\n");
          else
            print_output_buffer(cp, "NO\n");
	  return 1;
	}
    }
  else if (strncasecmp(data, "LOAD_MODULE", strlen("LOAD_MODULE")) == 0)
    {
      /* A period '.' as a version means to pass NULL; an empty string
       * means to pass an empty string
       */
      char *c = data + strlen("LOAD_MODULE"), *d = NULL;
      if (*c)
        {
          c++;
	  d = strchr(c, ' ');
        }
      if (!d)
	return -1;
      else
	{
          *d++ = '\0';
          load_module(c, (*d != '.') ? d : NULL);
	  return 1;
	}
    }
  else if (strncasecmp(data, "UNLOAD_MODULE", strlen("UNLOAD_MODULE")) == 0)
    {
      char *c = data + strlen("UNLOAD_MODULE");
      if (c)
        c++;
      if (!c)
	return -1;
      else
	{
          unload_module(c);
	  return 1;
	}
    }
  else if (strncasecmp(data, "REGISTER_HANDLER", strlen("REGISTER_HANDLER")) == 0)
    {
      char *c = data + strlen("REGISTER_HANDLER"), *d = NULL;
      if (*c)
        {
          c++;
	  d = strchr(c, ' ');
        }
      if (!d)
	return -1;
      else
	{
          struct module *m;
          *d++ = '\0';
          m = find_module(c);
          if (!m)
            return 0;
          else
            {
              register_module_unload_handler(MYHANDLE, m, &unload_handler, xstrdup(d));
              return 1;
            }
	}
    }
  else
    return 0;
}

static void
pipe_fail(void)
{
  unload_module(MYNAME);
}

int
MODULE_INIT(void *data unused_attribute)
{
  if (!(cp = setup_cmdpipe(MYHANDLE, MYNAME, 32768, &parse_command, &pipe_fail)))
    return 0;

  return MODULE_LOAD_OK;
}

int
MODULE_DEINIT(const char *new_version unused_attribute, module_init_func *init unused_attribute)
{
  if (cp)
    close_cmdpipe(cp);
  cp = NULL;
  return MODULE_UNLOAD_OK;
}
