/************************************************************************
 *   This is part of Dancer, a text relay daemon
 *   Copyright (C) 2002 Andrew Suffield <asuffield@users.sourceforge.net>
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 */

#include <dancerd/config.h>
#include <dancerd/modules.h>
#include <dancerd/log.h>

#include <stddef.h>

SETUP_MODULE("test.stub.b", MODVERSION);
RELOAD_INTERFACE(0);
MODULE_DEPS =
  {
    {"test.stub.a", "1.1"},
    {NULL, NULL}
  };

int
MODULE_INIT(void *data)
{
  log(MYNAME, LOG_PRIORITY_DEBUG, "MODULE_INIT called, data is %s", data ? "not null" : "null");
  if (data)
    {
      log(MYNAME, LOG_PRIORITY_ERROR, "Passed unknown data when initialising, aborting");
      return MODULE_LOAD_FAILED;
    }
  else
    return MODULE_LOAD_OK;
}

int
MODULE_DEINIT(const char *new_version, module_init_func *init)
{
  int ret = MODULE_UNLOAD_OK;

  log(MYNAME, LOG_PRIORITY_DEBUG, "MODULE_DEINIT called, new_version == %s", new_version);
  if (new_version)
    {
      switch ((*init)(NULL))
        {
        case MODULE_LOAD_OK:
          ret = MODULE_LOAD_OK;
          break;
        case MODULE_LOAD_FAILED:
          return MODULE_LOAD_FAILED;
        default:
          log(MYNAME, LOG_PRIORITY_ERROR, "Unknown return from new module version %s, aborting", new_version);
          return MODULE_LOAD_FAILED;
        }
    }

  return ret;
}
